# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.tagging import ansible_dict_to_boto3_tag_list
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags


@AWSRetry.jittered_backoff()
def _list_tags(wafv2, arn, fail_json_aws, next_marker=None):
    params = dict(ResourceARN=arn)
    if next_marker:
        params["NextMarker"] = next_marker
    try:
        return wafv2.list_tags_for_resource(**params)
    except (BotoCoreError, ClientError) as e:
        fail_json_aws(e, msg="Failed to list wafv2 tags")


def describe_wafv2_tags(wafv2, arn, fail_json_aws):
    next_marker = None
    tag_list = []
    # there is currently no paginator for wafv2
    while True:
        responce = _list_tags(wafv2, arn, fail_json_aws)
        next_marker = responce.get("NextMarker", None)
        tag_info = responce.get("TagInfoForResource", {})
        tag_list.extend(tag_info.get("TagList", []))
        if not next_marker:
            break
    return boto3_tag_list_to_ansible_dict(tag_list)


def ensure_wafv2_tags(wafv2, arn, tags, purge_tags, fail_json_aws, check_mode):
    if tags is None:
        return False

    current_tags = describe_wafv2_tags(wafv2, arn, fail_json_aws)
    tags_to_add, tags_to_remove = compare_aws_tags(current_tags, tags, purge_tags)
    if not tags_to_add and not tags_to_remove:
        return False

    if check_mode:
        return True

    if tags_to_add:
        try:
            boto3_tags = ansible_dict_to_boto3_tag_list(tags_to_add)
            wafv2.tag_resource(ResourceARN=arn, Tags=boto3_tags)
        except (BotoCoreError, ClientError) as e:
            fail_json_aws(e, msg="Failed to add wafv2 tags")
    if tags_to_remove:
        try:
            wafv2.untag_resource(ResourceARN=arn, TagKeys=tags_to_remove)
        except (BotoCoreError, ClientError) as e:
            fail_json_aws(e, msg="Failed to remove wafv2 tags")

    return True


def wafv2_list_web_acls(wafv2, scope, fail_json_aws, nextmarker=None):
    # there is currently no paginator for wafv2
    req_obj = {"Scope": scope, "Limit": 100}
    if nextmarker:
        req_obj["NextMarker"] = nextmarker

    try:
        response = wafv2.list_web_acls(**req_obj)
    except (BotoCoreError, ClientError) as e:
        fail_json_aws(e, msg="Failed to list wafv2 web acl")

    if response.get("NextMarker"):
        response["WebACLs"] += wafv2_list_web_acls(
            wafv2, scope, fail_json_aws, nextmarker=response.get("NextMarker")
        ).get("WebACLs")
    return response


def wafv2_list_rule_groups(wafv2, scope, fail_json_aws, nextmarker=None):
    # there is currently no paginator for wafv2
    req_obj = {"Scope": scope, "Limit": 100}
    if nextmarker:
        req_obj["NextMarker"] = nextmarker

    try:
        response = wafv2.list_rule_groups(**req_obj)
    except (BotoCoreError, ClientError) as e:
        fail_json_aws(e, msg="Failed to list wafv2 rule group")

    if response.get("NextMarker"):
        response["RuleGroups"] += wafv2_list_rule_groups(
            wafv2, scope, fail_json_aws, nextmarker=response.get("NextMarker")
        ).get("RuleGroups")
    return response


def wafv2_snake_dict_to_camel_dict(a):
    if not isinstance(a, dict):
        return a

    retval = {}
    for item in a.keys():
        if isinstance(a.get(item), dict):
            if "Ip" in item:
                retval[item.replace("Ip", "IP")] = wafv2_snake_dict_to_camel_dict(a.get(item))
            elif "Arn" == item:
                retval["ARN"] = wafv2_snake_dict_to_camel_dict(a.get(item))
            else:
                retval[item] = wafv2_snake_dict_to_camel_dict(a.get(item))
        elif isinstance(a.get(item), list):
            retval[item] = []
            for idx in range(len(a.get(item))):
                retval[item].append(wafv2_snake_dict_to_camel_dict(a.get(item)[idx]))
        elif "Ip" in item:
            retval[item.replace("Ip", "IP")] = a.get(item)
        elif "Arn" == item:
            retval["ARN"] = a.get(item)
        else:
            retval[item] = a.get(item)
    return retval


def nested_byte_values_to_strings(rule, keyname):
    """
    currently valid nested byte values in statements array are
        - OrStatement
        - AndStatement
        - NotStatement
    """
    if rule.get("Statement", {}).get(keyname):
        for idx in range(len(rule.get("Statement", {}).get(keyname, {}).get("Statements"))):
            if rule["Statement"][keyname]["Statements"][idx].get("ByteMatchStatement"):
                rule["Statement"][keyname]["Statements"][idx]["ByteMatchStatement"]["SearchString"] = (
                    rule.get("Statement")
                    .get(keyname)
                    .get("Statements")[idx]
                    .get("ByteMatchStatement")
                    .get("SearchString")
                    .decode("utf-8")
                )

    return rule


def byte_values_to_strings_before_compare(rules):
    for idx in range(len(rules)):
        if rules[idx].get("Statement", {}).get("ByteMatchStatement", {}).get("SearchString"):
            rules[idx]["Statement"]["ByteMatchStatement"]["SearchString"] = (
                rules[idx].get("Statement").get("ByteMatchStatement").get("SearchString").decode("utf-8")
            )

        else:
            for statement in ["AndStatement", "OrStatement", "NotStatement"]:
                if rules[idx].get("Statement", {}).get(statement):
                    rules[idx] = nested_byte_values_to_strings(rules[idx], statement)

    return rules


def compare_priority_rules(existing_rules, requested_rules, purge_rules, state):
    diff = False
    existing_rules = sorted(existing_rules, key=lambda k: k["Priority"])
    existing_rules = byte_values_to_strings_before_compare(existing_rules)
    requested_rules = sorted(requested_rules, key=lambda k: k["Priority"])

    if purge_rules and state == "present":
        merged_rules = requested_rules
        if len(existing_rules) == len(requested_rules):
            for idx in range(len(existing_rules)):
                if existing_rules[idx] != requested_rules[idx]:
                    diff = True
                    break
        else:
            diff = True

    else:
        # find same priority rules
        #   * pop same priority rule from existing rule
        #   * compare existing rule
        merged_rules = []
        ex_idx_pop = []
        for existing_idx in range(len(existing_rules)):
            for requested_idx in range(len(requested_rules)):
                if existing_rules[existing_idx].get("Priority") == requested_rules[requested_idx].get("Priority"):
                    if state == "present":
                        ex_idx_pop.append(existing_idx)
                        if existing_rules[existing_idx] != requested_rules[requested_idx]:
                            diff = True
                    elif existing_rules[existing_idx] == requested_rules[requested_idx]:
                        ex_idx_pop.append(existing_idx)
                        diff = True

        prev_count = len(existing_rules)
        for idx in ex_idx_pop:
            existing_rules.pop(idx)

        if state == "present":
            merged_rules = existing_rules + requested_rules

            if len(merged_rules) != prev_count:
                diff = True
        else:
            merged_rules = existing_rules

    return diff, merged_rules



#
# Copyright (c) 2025, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: binary_file
author: Felix Fontein (@felixfontein)
short_description: Read binary file and return it Base64 encoded
version_added: 11.2.0
description:
  - This lookup returns the contents from a file on the Ansible controller's file system.
  - The file is read as a binary file and its contents are returned Base64 encoded.
    This is similar to using P(ansible.builtin.file#lookup) combined with P(ansible.builtin.b64encode#filter),
    except that P(ansible.builtin.file#lookup) does not support binary files as it interprets the contents as UTF-8,
    which can cause the wrong content being Base64 encoded.
options:
  _terms:
    description:
      - Paths of the files to read.
      - Relative paths will be searched for in different places. See R(Ansible task paths, playbook_task_paths) for more details.
    required: true
    type: list
    elements: str
  not_exist:
    description:
      - Determine how to react if the specified file cannot be found.
    type: str
    choices:
      error: Raise an error.
      empty: Return an empty string for the file.
      empty_str:
        - Return the string C(empty) for the file.
        - This cannot be confused with Base64 encoding due to the missing padding.
    default: error
notes:
  - This lookup does not understand 'globbing' - use the P(ansible.builtin.fileglob#lookup) lookup instead.
seealso:
  - plugin: ansible.builtin.b64decode
    plugin_type: filter
    description: >-
      The b64decode filter can be used to decode Base64 encoded data.
      Note that Ansible cannot handle binary data, the data will be interpreted as UTF-8 text!
  - plugin: ansible.builtin.file
    plugin_type: lookup
    description: You can use this lookup plugin to read text files from the Ansible controller.
  - module: ansible.builtin.slurp
    description: >-
      Also allows to read binary files Base64 encoded, but from remote targets.
      With C(delegate_to: localhost) can be redirected to run on the controller, but you have to know the path to the file to read.
      Both this plugin and P(ansible.builtin.file#lookup) use some search path logic to for example also find files in the C(files)
      directory of a role.
  - ref: playbook_task_paths
    description: Search paths used for relative files.
"""

EXAMPLES = r"""
---
- name: Output Base64 contents of binary files on screen
  ansible.builtin.debug:
    msg: "Content: {{ lookup('community.general.binary_file', item) }}"
  loop:
    - some-binary-file.bin
"""

RETURN = r"""
_raw:
  description:
    - Base64 encoded content of requested files, or an empty string resp. the string C(empty), depending on the O(not_exist) option.
    - This list contains one string per element of O(_terms) in the same order as O(_terms).
  type: list
  elements: str
  returned: success
"""

import base64

from ansible.errors import AnsibleLookupError
from ansible.plugins.lookup import LookupBase
from ansible.utils.display import Display

from ansible_collections.community.general.plugins.plugin_utils._lookup import check_for_wrong_terms

display = Display()


class LookupModule(LookupBase):
    def run(self, terms, variables=None, **kwargs):
        self.set_options(var_options=variables, direct=kwargs)
        check_for_wrong_terms(self, direct=kwargs)
        not_exist = self.get_option("not_exist")

        result = []
        for term in terms:
            display.debug(f"Searching for binary file: {term!r}")
            path = self.find_file_in_search_path(variables, "files", term, ignore_missing=(not_exist != "error"))
            display.vvvv(f"community.general.binary_file lookup using {path} as file")

            if not path:
                if not_exist == "empty":
                    result.append("")
                    continue
                if not_exist == "empty_str":
                    result.append("empty")
                    continue
                raise AnsibleLookupError(f"Could not locate file in community.general.binary_file lookup: {term}")

            try:
                with open(path, "rb") as f:
                    result.append(base64.b64encode(f.read()).decode("utf-8"))
            except Exception as exc:
                raise AnsibleLookupError(f"Error while reading {path}: {exc}") from exc

        return result



# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible_collections.ansible.mysql.plugins.module_utils.version import LooseVersion
from ansible_collections.ansible.mysql.plugins.module_utils.mysql import get_server_version


def supports_roles(cursor):
    version = get_server_version(cursor)

    return LooseVersion(version) >= LooseVersion('8')


def is_mariadb():
    return False



# Copyright (C) 2020 Orion Poplawski <orion@nwra.com>
# Copyright (c) 2020 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
author: Orion Poplawski (@opoplawski)
name: cobbler
short_description: Cobbler inventory source
version_added: 1.0.0
description:
  - Get inventory hosts from the cobbler service.
  - 'Uses a configuration file as an inventory source, it must end in C(.cobbler.yml) or C(.cobbler.yaml) and have a C(plugin:
    cobbler) entry.'
  - Adds the primary IP addresses to C(cobbler_ipv4_address) and C(cobbler_ipv6_address) host variables if defined in Cobbler.
    The primary IP address is defined as the management interface if defined, or the interface who's DNS name matches the
    hostname of the system, or else the first interface found.
extends_documentation_fragment:
  - ansible.builtin.inventory_cache
options:
  plugin:
    description: The name of this plugin, it should always be set to V(community.general.cobbler) for this plugin to recognize
      it as its own.
    type: string
    required: true
    choices: ['cobbler', 'community.general.cobbler']
  url:
    description: URL to cobbler.
    type: string
    default: 'http://cobbler/cobbler_api'
    env:
      - name: COBBLER_SERVER
  user:
    description: Cobbler authentication user.
    type: string
    required: false
    env:
      - name: COBBLER_USER
  password:
    description: Cobbler authentication password.
    type: string
    required: false
    env:
      - name: COBBLER_PASSWORD
  cache_fallback:
    description: Fallback to cached results if connection to cobbler fails.
    type: boolean
    default: false
  connection_timeout:
    description: Timeout to connect to cobbler server.
    type: int
    required: false
    version_added: 10.7.0
  exclude_mgmt_classes:
    description: Management classes to exclude from inventory.
    type: list
    default: []
    elements: str
    version_added: 7.4.0
  exclude_profiles:
    description:
      - Profiles to exclude from inventory.
      - Ignored if O(include_profiles) is specified.
    type: list
    default: []
    elements: str
  include_mgmt_classes:
    description: Management classes to include from inventory.
    type: list
    default: []
    elements: str
    version_added: 7.4.0
  include_profiles:
    description:
      - Profiles to include from inventory.
      - If specified, all other profiles are excluded.
      - O(exclude_profiles) is ignored if O(include_profiles) is specified.
    type: list
    default: []
    elements: str
    version_added: 4.4.0
  inventory_hostname:
    description:
      - What to use for the ansible inventory hostname.
      - By default the networking hostname is used if defined, otherwise the DNS name of the management or first non-static
        interface.
      - If set to V(system), the cobbler system name is used.
    type: str
    choices: ['hostname', 'system']
    default: hostname
    version_added: 7.1.0
  group_by:
    description: Keys to group hosts by.
    type: list
    elements: string
    default: ['mgmt_classes', 'owners', 'status']
  group:
    description: Group to place all hosts into.
    default: cobbler
  group_prefix:
    description: Prefix to apply to cobbler groups.
    default: cobbler_
  want_facts:
    description: Toggle, if V(true) the plugin retrieves all host facts from the server.
    type: boolean
    default: true
  want_ip_addresses:
    description:
      - Toggle, if V(true) the plugin adds a C(cobbler_ipv4_addresses) and C(cobbler_ipv6_addresses) dictionary to the
        defined O(group) mapping interface DNS names to IP addresses.
    type: boolean
    default: true
    version_added: 7.1.0
  facts_level:
    description:
      - Set to V(normal) to gather only system-level variables.
      - Set to V(as_rendered) to gather all variables as rolled up by Cobbler.
    type: string
    choices: ['normal', 'as_rendered']
    default: normal
    version_added: 10.7.0
"""

EXAMPLES = r"""
# my.cobbler.yml
plugin: community.general.cobbler
url: http://cobbler/cobbler_api
user: ansible-tester
password: secure
"""

import socket

from ansible.errors import AnsibleError
from ansible.plugins.inventory import BaseInventoryPlugin, Cacheable, to_safe_group_name

from ansible_collections.community.general.plugins.plugin_utils._unsafe import make_unsafe

# xmlrpc
try:
    import xmlrpc.client as xmlrpc_client

    HAS_XMLRPC_CLIENT = True
except ImportError:
    HAS_XMLRPC_CLIENT = False


class TimeoutTransport(xmlrpc_client.SafeTransport):
    def __init__(self, timeout=socket._GLOBAL_DEFAULT_TIMEOUT):
        super().__init__()
        self._timeout = timeout
        self.context = None

    def make_connection(self, host):
        conn = xmlrpc_client.SafeTransport.make_connection(self, host)
        conn.timeout = self._timeout
        return conn


class InventoryModule(BaseInventoryPlugin, Cacheable):
    """Host inventory parser for ansible using cobbler as source."""

    NAME = "community.general.cobbler"

    def __init__(self):
        super().__init__()
        self.cache_key = None

        if not HAS_XMLRPC_CLIENT:
            raise AnsibleError("Could not import xmlrpc client library")

    def verify_file(self, path):
        valid = False
        if super().verify_file(path):
            if path.endswith(("cobbler.yaml", "cobbler.yml")):
                valid = True
            else:
                self.display.vvv('Skipping due to inventory source not ending in "cobbler.yaml" nor "cobbler.yml"')
        return valid

    def _init_cache(self):
        if self.cache_key not in self._cache:
            self._cache[self.cache_key] = {}

    def _reload_cache(self):
        if self.get_option("cache_fallback"):
            self.display.vvv("Cannot connect to server, loading cache\n")
            self._options["cache_timeout"] = 0
            self.load_cache_plugin()
            self._cache.get(self.cache_key, {})

    def _get_profiles(self):
        if not self.use_cache or "profiles" not in self._cache.get(self.cache_key, {}):
            try:
                if self.token is not None:
                    data = self.cobbler.get_profiles(self.token)
                else:
                    data = self.cobbler.get_profiles()
            except (socket.gaierror, OSError, xmlrpc_client.ProtocolError):
                self._reload_cache()
            else:
                self._init_cache()
                self._cache[self.cache_key]["profiles"] = data

        return self._cache[self.cache_key]["profiles"]

    def _get_systems(self):
        if not self.use_cache or "systems" not in self._cache.get(self.cache_key, {}):
            try:
                if self.token is not None:
                    data = self.cobbler.get_systems(self.token)
                else:
                    data = self.cobbler.get_systems()

                # If more facts are requested, gather them all from Cobbler
                if self.facts_level == "as_rendered":
                    for i, host in enumerate(data):
                        self.display.vvvv(f"Gathering all facts for {host['name']}\n")
                        if self.token is not None:
                            data[i] = self.cobbler.get_system_as_rendered(host["name"], self.token)
                        else:
                            data[i] = self.cobbler.get_system_as_rendered(host["name"])
            except (socket.gaierror, OSError, xmlrpc_client.ProtocolError):
                self._reload_cache()
            else:
                self._init_cache()
                self._cache[self.cache_key]["systems"] = data

        return self._cache[self.cache_key]["systems"]

    def _add_safe_group_name(self, group, child=None):
        group_name = self.inventory.add_group(
            to_safe_group_name(f"{self.get_option('group_prefix')}{group.lower().replace(' ', '')}")
        )
        if child is not None:
            self.inventory.add_child(group_name, child)
        return group_name

    def _exclude_profile(self, profile):
        if self.include_profiles:
            return profile not in self.include_profiles
        else:
            return profile in self.exclude_profiles

    def parse(self, inventory, loader, path, cache=True):
        super().parse(inventory, loader, path)

        # read config from file, this sets 'options'
        self._read_config_data(path)

        # get connection host
        self.cobbler_url = self.get_option("url")
        self.display.vvvv(f"Connecting to {self.cobbler_url}\n")

        if "connection_timeout" in self._options:
            self.cobbler = xmlrpc_client.Server(
                self.cobbler_url,
                allow_none=True,
                transport=TimeoutTransport(timeout=self.get_option("connection_timeout")),
            )
        else:
            self.cobbler = xmlrpc_client.Server(self.cobbler_url, allow_none=True)
        self.token = None
        if self.get_option("user") is not None:
            self.token = self.cobbler.login(str(self.get_option("user")), str(self.get_option("password")))

        self.cache_key = self.get_cache_key(path)
        self.use_cache = cache and self.get_option("cache")

        self.exclude_mgmt_classes = self.get_option("exclude_mgmt_classes")
        self.include_mgmt_classes = self.get_option("include_mgmt_classes")
        self.exclude_profiles = self.get_option("exclude_profiles")
        self.include_profiles = self.get_option("include_profiles")
        self.group_by = self.get_option("group_by")
        self.inventory_hostname = self.get_option("inventory_hostname")
        self.facts_level = self.get_option("facts_level")

        for profile in self._get_profiles():
            if profile["parent"]:
                self.display.vvvv(f"Processing profile {profile['name']} with parent {profile['parent']}\n")
                if not self._exclude_profile(profile["parent"]):
                    parent_group_name = self._add_safe_group_name(profile["parent"])
                    self.display.vvvv(f"Added profile parent group {parent_group_name}\n")
                    if not self._exclude_profile(profile["name"]):
                        group_name = self._add_safe_group_name(profile["name"])
                        self.display.vvvv(f"Added profile group {group_name}\n")
                        self.inventory.add_child(parent_group_name, group_name)
            else:
                self.display.vvvv(f"Processing profile {profile['name']} without parent\n")
                # Create a hierarchy of profile names
                profile_elements = profile["name"].split("-")
                i = 0
                while i < len(profile_elements) - 1:
                    profile_group = "-".join(profile_elements[0 : i + 1])
                    profile_group_child = "-".join(profile_elements[0 : i + 2])
                    if self._exclude_profile(profile_group):
                        self.display.vvvv(f"Excluding profile {profile_group}\n")
                        break
                    group_name = self._add_safe_group_name(profile_group)
                    self.display.vvvv(f"Added profile group {group_name}\n")
                    child_group_name = self._add_safe_group_name(profile_group_child)
                    self.display.vvvv(f"Added profile child group {child_group_name} to {group_name}\n")
                    self.inventory.add_child(group_name, child_group_name)
                    i = i + 1

        # Add default group for this inventory if specified
        self.group = to_safe_group_name(self.get_option("group"))
        if self.group is not None and self.group != "":
            self.inventory.add_group(self.group)
            self.display.vvvv(f"Added site group {self.group}\n")

        ip_addresses = {}
        ipv6_addresses = {}
        for host in self._get_systems():
            # Get the FQDN for the host and add it to the right groups
            if self.inventory_hostname == "system":
                hostname = make_unsafe(host["name"])  # None
            else:
                hostname = make_unsafe(host["hostname"])  # None
            interfaces = host["interfaces"]

            if set(host["mgmt_classes"]) & set(self.include_mgmt_classes):
                self.display.vvvv(f"Including host {host['name']} in mgmt_classes {host['mgmt_classes']}\n")
            else:
                if self._exclude_profile(host["profile"]):
                    self.display.vvvv(f"Excluding host {host['name']} in profile {host['profile']}\n")
                    continue

                if set(host["mgmt_classes"]) & set(self.exclude_mgmt_classes):
                    self.display.vvvv(f"Excluding host {host['name']} in mgmt_classes {host['mgmt_classes']}\n")
                    continue

            # hostname is often empty for non-static IP hosts
            if hostname == "":
                for iname, ivalue in interfaces.items():
                    if ivalue["management"] or not ivalue["static"]:
                        this_dns_name = ivalue.get("dns_name", None)
                        if this_dns_name is not None and this_dns_name != "":
                            hostname = make_unsafe(this_dns_name)
                            self.display.vvvv(f"Set hostname to {hostname} from {iname}\n")

            if hostname == "":
                self.display.vvvv(f"Cannot determine hostname for host {host['name']}, skipping\n")
                continue

            self.inventory.add_host(hostname)
            self.display.vvvv(f"Added host {host['name']} hostname {hostname}\n")

            # Add host to profile group
            if host["profile"] != "":
                group_name = self._add_safe_group_name(host["profile"], child=hostname)
                self.display.vvvv(f"Added host {hostname} to profile group {group_name}\n")
            else:
                self.display.warning(f"Host {hostname} has an empty profile\n")

            # Add host to groups specified by group_by fields
            for group_by in self.group_by:
                if host[group_by] == "<<inherit>>" or host[group_by] == "":
                    groups = []
                else:
                    groups = [host[group_by]] if isinstance(host[group_by], str) else host[group_by]
                for group in groups:
                    group_name = self._add_safe_group_name(group, child=hostname)
                    self.display.vvvv(f"Added host {hostname} to group_by {group_by} group {group_name}\n")

            # Add to group for this inventory
            if self.group is not None:
                self.inventory.add_child(self.group, hostname)

            # Add host variables
            ip_address = None
            ip_address_first = None
            ipv6_address = None
            ipv6_address_first = None
            for ivalue in interfaces.values():
                # Set to first interface or management interface if defined or hostname matches dns_name
                if ivalue["ip_address"] != "":
                    if ip_address_first is None:
                        ip_address_first = ivalue["ip_address"]
                    if ivalue["management"]:
                        ip_address = ivalue["ip_address"]
                    elif ivalue["dns_name"] == hostname and ip_address is None:
                        ip_address = ivalue["ip_address"]
                if ivalue["ipv6_address"] != "":
                    if ipv6_address_first is None:
                        ipv6_address_first = ivalue["ipv6_address"]
                    if ivalue["management"]:
                        ipv6_address = ivalue["ipv6_address"]
                    elif ivalue["dns_name"] == hostname and ipv6_address is None:
                        ipv6_address = ivalue["ipv6_address"]

                # Collect all interface name mappings for adding to group vars
                if self.get_option("want_ip_addresses"):
                    if ivalue["dns_name"] != "":
                        if ivalue["ip_address"] != "":
                            ip_addresses[ivalue["dns_name"]] = ivalue["ip_address"]
                        if ivalue["ipv6_address"] != "":
                            ip_addresses[ivalue["dns_name"]] = ivalue["ipv6_address"]

            # Add ip_address to host if defined, use first if no management or matched dns_name
            if ip_address is None and ip_address_first is not None:
                ip_address = ip_address_first
            if ip_address is not None:
                self.inventory.set_variable(hostname, "cobbler_ipv4_address", make_unsafe(ip_address))
            if ipv6_address is None and ipv6_address_first is not None:
                ipv6_address = ipv6_address_first
            if ipv6_address is not None:
                self.inventory.set_variable(hostname, "cobbler_ipv6_address", make_unsafe(ipv6_address))

            if self.get_option("want_facts"):
                try:
                    self.inventory.set_variable(hostname, "cobbler", make_unsafe(host))
                except ValueError as e:
                    self.display.warning(f"Could not set host info for {hostname}: {e}")

        if self.get_option("want_ip_addresses"):
            self.inventory.set_variable(self.group, "cobbler_ipv4_addresses", make_unsafe(ip_addresses))
            self.inventory.set_variable(self.group, "cobbler_ipv6_addresses", make_unsafe(ipv6_addresses))



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_online_deployment_info
version_added: "3.17.0"
short_description: List ML Online Deployment resources
description:
    - List ML Online Deployment resources.
options:
    name:
        description:
            - Name of the deployment.
        type: str
        required: false
    endpoint_name:
        description:
            - Name of the endpoint.
        required: true
        type: str
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: true
        type: str
    local:
        description:
            - List all local deployments.
        type: bool
        default: false

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Get Specific ML Online Deployment by name
  azure.azcollection.azure_rm_ml_online_deployment_info:
    name: deployment-command-xxxxxxxxxx
    endpoint_name: endpoint-command-xxxxxxxxxx
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace

- name: List Local ML Online Deployments
  azure.azcollection.azure_rm_ml_online_deployment_info:
    endpoint_name: endpoint-command-xxxxxxxxxx
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace
    local: true
'''

RETURN = '''
ml_online_deployments:
    description:
        - Online Deployments that match the query.
    returned: always
    type: dict
    sample: [
      {
          "app_insights_enabled": false,
          "code_configuration": {
              "code": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-online_deployment/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/codes/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/versions/1",
              "scoring_script": "score.py"
          },
          "creation_context": {
              "created_at": "2026-04-17T20:39:38.864149+00:00",
              "created_by": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
              "last_modified_at": "2026-04-17T20:39:38.864151+00:00"
          },
          "egress_public_network_access": "enabled",
          "endpoint_name": "endpoint-command-ansible",
          "environment": "azureml:/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-online_deployment/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/environments/CliV2AnonymousEnvironment/versions/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
          "environment_variables": {
              "AML_APP_ROOT": "/var/azureml-app/onlinescoring",
              "AZUREML_ENTRY_SCRIPT": "score.py",
              "AZUREML_MODEL_DIR": "/var/azureml-app/azureml-models/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/1"
          },
          "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-online_deployment/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/onlineEndpoints/endpoint-command-ansible/deployments/deployment-command-xxxxxxxxxx",
          "instance_count": 1,
          "instance_type": "Standard_DS3_v2",
          "liveness_probe": {
              "failure_threshold": 30,
              "initial_delay": 10,
              "period": 10,
              "success_threshold": 1,
              "timeout": 2
          },
          "model": "azureml:/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-online_deployment/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/models/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/versions/1",
          "name": "deployment-command-e76e189747",
          "properties": {
              "AzureAsyncOperationUri": "https://management.azure.com/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/providers/Microsoft.MachineLearningServices/locations/eastus/mfeOperationsStatus/odidp:xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx:xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx?api-version=2023-04-01-preview"
          },
          "provisioning_state": "Succeeded",
          "readiness_probe": {
              "failure_threshold": 30,
              "initial_delay": 10,
              "period": 10,
              "success_threshold": 1,
              "timeout": 2
          },
          "request_settings": {
              "max_concurrent_requests_per_instance": 1,
              "request_timeout_ms": 5000
          },
          "scale_settings": {
              "type": "default"
          },
          "tags": {},
          "type": "managed"
      }
    ]
'''  # NOQA


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLOnlineDeploymentInfo(MLClientCommon):
    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=False,
            ),
            endpoint_name=dict(
                type='str',
                required=True,
            ),
            resource_group=dict(
                type='str',
                required=True,
            ),
            ml_workspace=dict(
                type='str',
                required=True,
            ),
            local=dict(
                type='bool',
                default=False,
            ),
        )

        self._client = None
        self.ml_registry = None

        self.results = dict(
            ml_online_deployments=[]
        )

        super(AzureRMMLOnlineDeploymentInfo, self).__init__(self.module_arg_spec,
                                                            supports_tags=False,
                                                            supports_check_mode=True,
                                                            facts_module=True,
                                                            )

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        if self.name:
            ml_online_deployments = [self.get(self.name,
                                              endpoint_name=self.endpoint_name,
                                              local=self.local)]
        else:
            results = self.client.online_deployments.list(endpoint_name=self.endpoint_name,
                                                          local=self.local)
            ml_online_deployments = [
                self.get(x.name,
                         endpoint_name=self.endpoint_name,
                         local=self.local)
                for x in results]

        self.results['ml_online_deployments'] = ml_online_deployments
        return self.results

    def get(self, name, endpoint_name=None, local=False):
        try:
            result = self.client.online_deployments.get(name=name,
                                                        endpoint_name=endpoint_name,
                                                        local=self.local)
            ml_online_deployment = self.entity_to_dict(result)
        except ResourceNotFoundError:
            ml_online_deployment = None

        return ml_online_deployment


def main():
    AzureRMMLOnlineDeploymentInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_storage_policy_info
short_description: Gather information about Storage Policies in Cisco Intersight
description:
  - Gather information about Storage Policies in L(Cisco Intersight,https://intersight.com).
  - Information can be filtered by O(organization) and O(name).
  - If no filters are passed, all Storage Policies will be returned.
extends_documentation_fragment: intersight
options:
  organization:
    description:
      - The name of the organization the Storage Policy belongs to.
    type: str
  name:
    description:
      - The name of the Storage Policy to gather information from.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Fetch a specific Storage Policy by name
  cisco.intersight.intersight_storage_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "basic-storage-policy"

- name: Fetch all Storage Policies in a specific Organization
  cisco.intersight.intersight_storage_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"

- name: Fetch all Storage Policies
  cisco.intersight.intersight_storage_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": [
    {
        "Name": "basic-storage-policy",
        "ObjectType": "storage.StoragePolicy",
        "UseJbodForVdCreation": true,
        "UnusedDisksState": "NoChange",
        "DefaultDriveMode": "UnconfiguredGood",
        "ControllerAttachedNvmeSlots": "",
        "DirectAttachedNvmeSlots": "",
        "SecureJbods": "",
        "M2VirtualDrive": {
            "Enable": false,
            "ControllerSlot": "MSTOR-RAID-1",
            "Name": "MStorBootVd"
        },
        "Raid0Drive": {
            "Enable": false,
            "DriveSlots": "",
            "VirtualDrivePolicy": {
                "StripSize": 64,
                "AccessPolicy": "Default",
                "ReadPolicy": "Default",
                "WritePolicy": "Default",
                "DriveCache": "Default"
            }
        },
        "DriveGroups": [
            {
                "Name": "raid0-group",
                "RaidLevel": "Raid0",
                "SecureDriveGroup": false,
                "ManualDriveGroup": {
                    "SpanGroups": [
                        {
                            "Slots": "1,2"
                        }
                    ]
                },
                "VirtualDrives": [
                    {
                        "Name": "raid0-vd",
                        "Size": 1024,
                        "ExpandToAvailable": false,
                        "BootDrive": true,
                        "VirtualDrivePolicy": {
                            "StripSize": 128,
                            "AccessPolicy": "ReadWrite",
                            "ReadPolicy": "ReadAhead",
                            "WritePolicy": "WriteBackGoodBbu",
                            "DriveCache": "Enable"
                        }
                    }
                ]
            }
        ],
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    },
    {
        "Name": "m2-enabled-storage-policy",
        "ObjectType": "storage.StoragePolicy",
        "UseJbodForVdCreation": true,
        "UnusedDisksState": "NoChange",
        "DefaultDriveMode": "UnconfiguredGood",
        "ControllerAttachedNvmeSlots": "",
        "DirectAttachedNvmeSlots": "",
        "SecureJbods": "",
        "M2VirtualDrive": {
            "Enable": true,
            "ControllerSlot": "MSTOR-RAID-1",
            "Name": "MStorBootVd"
        },
        "Raid0Drive": {
            "Enable": false,
            "DriveSlots": "",
            "VirtualDrivePolicy": {
                "StripSize": 64,
                "AccessPolicy": "Default",
                "ReadPolicy": "Default",
                "WritePolicy": "Default",
                "DriveCache": "Default"
            }
        },
        "Tags": [
            {
                "Key": "Site",
                "Value": "Datacenter1"
            }
        ]
    }
  ]
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        organization=dict(type='str'),
        name=dict(type='str')
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    resource_path = '/storage/StoragePolicies'

    query_params = intersight.set_query_params()

    intersight.get_resource(
        resource_path=resource_path,
        query_params=query_params,
        return_list=True
    )

    # Fetch drive groups for each storage policy
    storage_policies = intersight.result['api_response']
    if isinstance(storage_policies, list):
        for policy in storage_policies:
            if policy.get('Moid'):
                # Fetch drive groups for this policy using StoragePolicy.Moid filter
                drive_groups_query_params = {
                    '$filter': f"StoragePolicy.Moid eq '{policy['Moid']}'"
                }

                # Create a temporary intersight instance for drive groups query
                temp_intersight = IntersightModule(module)
                temp_intersight.get_resource(
                    resource_path='/storage/DriveGroups',
                    query_params=drive_groups_query_params,
                    return_list=True
                )

                # Add drive groups to the policy
                policy['DriveGroups'] = temp_intersight.result.get('api_response', [])
    elif isinstance(storage_policies, dict) and storage_policies.get('Moid'):
        # Single policy case
        drive_groups_query_params = {
            '$filter': f"StoragePolicy.Moid eq '{storage_policies['Moid']}'"
        }

        # Create a temporary intersight instance for drive groups query
        temp_intersight = IntersightModule(module)
        temp_intersight.get_resource(
            resource_path='/storage/DriveGroups',
            query_params=drive_groups_query_params,
            return_list=True
        )

        # Add drive groups to the policy
        storage_policies['DriveGroups'] = temp_intersight.result.get('api_response', [])

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ec2_customer_gateway
version_added: 1.0.0
short_description: Manage an AWS customer gateway
description:
  - Manage an AWS customer gateway.
author:
  - Michael Baydoun (@MichaelBaydoun)
notes:
  - You cannot create more than one customer gateway with the same IP address. If you run an identical request more than one time, the
    first request creates the customer gateway, and subsequent requests return information about the existing customer gateway. The subsequent
    requests do not create new customer gateway resources.
  - Return values contain customer_gateway and customer_gateways keys which are identical dicts. You should use
    customer_gateway. See U(https://github.com/ansible/ansible-modules-extras/issues/2773) for details.
options:
  bgp_asn:
    description:
      - Border Gateway Protocol (BGP) Autonomous System Number (ASN).
      - Defaults to C(65000) if not specified when I(state=present).
    type: int
  ip_address:
    description:
      - Internet-routable IP address for customers gateway, must be a static address.
    required: true
    type: str
  name:
    description:
      - Name of the customer gateway.
    required: true
    type: str
  routing:
    description:
      - The type of routing.
    choices: ['static', 'dynamic']
    default: dynamic
    type: str
  state:
    description:
      - Create or terminate the Customer Gateway.
    default: present
    choices: [ 'present', 'absent' ]
    type: str
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Create Customer Gateway
  community.aws.ec2_customer_gateway:
    bgp_asn: 12345
    ip_address: 1.2.3.4
    name: IndianapolisOffice
    region: us-east-1
  register: cgw

- name: Delete Customer Gateway
  community.aws.ec2_customer_gateway:
    ip_address: 1.2.3.4
    name: IndianapolisOffice
    state: absent
    region: us-east-1
  register: cgw
"""

RETURN = r"""
gateway:
    description: details about the customer gateway that was created.
    returned: success
    type: complex
    contains:
        customer_gateways:
            description: details about the gateway that was created.
            returned: success
            type: complex
            contains:
                bgp_asn:
                    description: The Border Gateway Autonomous System Number.
                    returned: when exists and gateway is available.
                    sample: 65123
                    type: str
                customer_gateway_id:
                    description: gateway id assigned by amazon.
                    returned: when exists and gateway is available.
                    sample: cgw-cb6386a2
                    type: str
                ip_address:
                    description: ip address of your gateway device.
                    returned: when exists and gateway is available.
                    sample: 1.2.3.4
                    type: str
                state:
                    description: state of gateway.
                    returned: when gateway exists and is available.
                    sample: available
                    type: str
                tags:
                    description: Any tags on the gateway.
                    returned: when gateway exists and is available, and when tags exist.
                    type: list
                type:
                    description: encryption type.
                    returned: when gateway exists and is available.
                    sample: ipsec.1
                    type: str
"""

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


class Ec2CustomerGatewayManager:
    def __init__(self, module):
        self.module = module

        try:
            self.ec2 = module.client("ec2")
        except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
            module.fail_json_aws(e, msg="Failed to connect to AWS")

    @AWSRetry.jittered_backoff(delay=2, max_delay=30, retries=6, catch_extra_error_codes=["IncorrectState"])
    def ensure_cgw_absent(self, gw_id):
        response = self.ec2.delete_customer_gateway(DryRun=False, CustomerGatewayId=gw_id)
        return response

    def ensure_cgw_present(self, bgp_asn, ip_address):
        if not bgp_asn:
            bgp_asn = 65000
        response = self.ec2.create_customer_gateway(
            DryRun=False,
            Type="ipsec.1",
            PublicIp=ip_address,
            BgpAsn=bgp_asn,
        )
        return response

    def tag_cgw_name(self, gw_id, name):
        response = self.ec2.create_tags(
            DryRun=False,
            Resources=[
                gw_id,
            ],
            Tags=[
                {"Key": "Name", "Value": name},
            ],
        )
        return response

    def describe_gateways(self, ip_address):
        response = self.ec2.describe_customer_gateways(
            DryRun=False,
            Filters=[
                {
                    "Name": "state",
                    "Values": [
                        "available",
                    ],
                },
                {
                    "Name": "ip-address",
                    "Values": [
                        ip_address,
                    ],
                },
            ],
        )
        return response


def main():
    argument_spec = dict(
        bgp_asn=dict(required=False, type="int"),
        ip_address=dict(required=True),
        name=dict(required=True),
        routing=dict(default="dynamic", choices=["dynamic", "static"]),
        state=dict(default="present", choices=["present", "absent"]),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ("routing", "dynamic", ["bgp_asn"]),
        ],
    )

    gw_mgr = Ec2CustomerGatewayManager(module)

    name = module.params.get("name")

    existing = gw_mgr.describe_gateways(module.params["ip_address"])

    results = dict(changed=False)
    if module.params["state"] == "present":
        if existing["CustomerGateways"]:
            existing["CustomerGateway"] = existing["CustomerGateways"][0]
            results["gateway"] = existing
            if existing["CustomerGateway"]["Tags"]:
                tag_array = existing["CustomerGateway"]["Tags"]
                for key, value in enumerate(tag_array):
                    if value["Key"] == "Name":
                        current_name = value["Value"]
                        if current_name != name:
                            results["name"] = gw_mgr.tag_cgw_name(
                                results["gateway"]["CustomerGateway"]["CustomerGatewayId"],
                                module.params["name"],
                            )
                            results["changed"] = True
        else:
            if not module.check_mode:
                results["gateway"] = gw_mgr.ensure_cgw_present(
                    module.params["bgp_asn"],
                    module.params["ip_address"],
                )
                results["name"] = gw_mgr.tag_cgw_name(
                    results["gateway"]["CustomerGateway"]["CustomerGatewayId"],
                    module.params["name"],
                )
            results["changed"] = True

    elif module.params["state"] == "absent":
        if existing["CustomerGateways"]:
            existing["CustomerGateway"] = existing["CustomerGateways"][0]
            results["gateway"] = existing
            if not module.check_mode:
                results["gateway"] = gw_mgr.ensure_cgw_absent(existing["CustomerGateway"]["CustomerGatewayId"])
            results["changed"] = True

    pretty_results = camel_dict_to_snake_dict(results)
    module.exit_json(**pretty_results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Sabari Jaganathan (@sajagana) <sajagana@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_l3out_node_routing_policy
short_description: Manage L3Out Node Routing Policies on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage L3Out Node Routing Policies on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Sabari Jaganathan (@sajagana)
options:
  template:
    description:
    - The name of the template.
    - The template must be a Tenant Policy Template.
    type: str
    required: true
  name:
    description:
    - The name of the L3Out Node Routing Policy.
    type: str
    aliases: [ l3out_node_routing_policy_name ]
  uuid:
    description:
    - The UUID of the L3Out Node Routing Policy.
    - This parameter is required when the O(name) needs to be updated.
    type: str
    aliases: [ l3out_node_routing_policy_uuid ]
  description:
    description:
    - The description of the L3Out Node Routing Policy.
    type: str
  bfd_multi_hop_settings:
    description:
    - The BFD MultiHop Settings configuration of the L3Out Node Routing Policy.
    type: dict
    suboptions:
      state:
        description:
        - Use C(enabled) to configure the BFD MultiHop Settings.
        - Use C(disabled) to remove the BFD MultiHop Settings.
        type: str
        choices: [ enabled, disabled ]
      admin_state:
        description:
        - The administrative state of the BFD MultiHop Settings.
        - Defaults to enabled when unset during creation.
        type: str
        choices: [ enabled, disabled ]
      detection_multiplier:
        description:
        - The detection multiplier of the BFD MultiHop Settings.
        - Defaults to 3 when unset during creation.
        - The value must be between 1 and 50.
        type: int
      min_receive_interval:
        description:
        - The minimum receive interval of the BFD MultiHop Settings.
        - Defaults to 250 when unset during creation.
        - The value must be between 250 and 999 microseconds.
        type: int
      min_transmit_interval:
        description:
        - The minimum transmit interval of the BFD MultiHop Settings.
        - Defaults to 250 when unset during creation.
        - The value must be between 250 and 999 microseconds.
        type: int
  bgp_node_settings:
    description:
    - The BGP Node Settings configuration of the L3Out Node Routing Policy.
    type: dict
    suboptions:
      state:
        description:
        - Use C(enabled) to configure the BGP Node Settings.
        - Use C(disabled) to remove the BGP Node Settings.
        type: str
        choices: [ enabled, disabled ]
      graceful_restart_helper:
        description:
        - The graceful restart helper of the BGP Node Settings.
        - Defaults to enabled when unset during creation.
        type: str
        choices: [ enabled, disabled ]
      keep_alive_interval:
        description:
        - The keep alive interval of the BGP Node Settings.
        - Defaults to 60 when unset during creation.
        - The value must be between 0 and 3600 seconds.
        type: int
      hold_interval:
        description:
        - The hold interval of the BGP Node Settings.
        - Defaults to 180 when unset during creation.
        - The value must be 0 or between 3 and 3600 seconds.
        type: int
      stale_interval:
        description:
        - The stale interval of the BGP Node Settings.
        - Defaults to 300 when unset during creation.
        - The value must be between 1 and 3600 seconds.
        type: int
      max_as_limit:
        description:
        - The max as limit of the BGP Node Settings.
        - Defaults to 0 when unset during creation.
        - The value must be between 0 and 2000.
        type: int
  as_path_multipath_relax:
    description:
    - The BGP Best Path Control of the L3Out Node Routing Policy.
    - Providing an empty string will remove the O(as_path_multipath_relax="") from the L3Out Node Routing Policy.
    type: str
    choices: [ enabled, disabled, "" ]
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the Tenant Policy Template.
seealso:
- module: cisco.mso.ndo_template
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new L3Out Node Routing Policy with empty bgp_node and bfd_multi_hop settings
  cisco.mso.ndo_l3out_node_routing_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    name: nrp_1
    bfd_multi_hop_settings:
      state: enabled
    bgp_node_settings:
      state: enabled
    as_path_multipath_relax: disabled
    state: present

- name: Create a new L3Out Node Routing Policy with full config of bgp_node and bfd_multi_hop settings
  cisco.mso.ndo_l3out_node_routing_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    name: nrp_2
    bfd_multi_hop_settings:
      admin_state: enabled
      detection_multiplier: 10
      min_receive_interval: 450
      min_transmit_interval: 550
    bgp_node_settings:
      graceful_restart_helper: enabled
      keep_alive_interval: 15
      hold_interval: 115
      stale_interval: 215
      max_as_limit: 25
    as_path_multipath_relax: disabled
    state: present
  register: nrp_2

- name: Update L3Out Node Routing Policy name and bgp_node_settings attributes
  cisco.mso.ndo_l3out_node_routing_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    uuid: "{{ nrp_2.current.uuid }}"
    name: nrp_2_updated
    bgp_node_settings:
      graceful_restart_helper: disabled
      keep_alive_interval: 20
    state: present

- name: Query a L3Out Node Routing Policies with name
  cisco.mso.ndo_l3out_node_routing_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    name: nrp_1
    state: query
  register: query_with_name

- name: Query a L3Out Node Routing Policies with UUID
  cisco.mso.ndo_l3out_node_routing_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    uuid: "{{ query_with_name.current.uuid }}"
    state: query
  register: query_with_uuid

- name: Query all L3Out Node Routing Policies
  cisco.mso.ndo_l3out_node_routing_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    state: query
  register: query_all

- name: Delete a L3Out Node Routing Policy
  cisco.mso.ndo_l3out_node_routing_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    name: nrp_1
    state: absent

- name: Delete a L3Out Node Routing Policy using UUID
  cisco.mso.ndo_l3out_node_routing_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    uuid: "{{ query_with_name.current.uuid }}"
    state: absent
"""

RETURN = r"""
"""


import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec, ndo_bfd_multi_hop_settings_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", required=True),
        name=dict(type="str", aliases=["l3out_node_routing_policy_name"]),
        uuid=dict(type="str", aliases=["l3out_node_routing_policy_uuid"]),
        description=dict(type="str"),
        bfd_multi_hop_settings=ndo_bfd_multi_hop_settings_spec(),
        bgp_node_settings=dict(
            type="dict",
            options=dict(
                state=dict(type="str", choices=["enabled", "disabled"]),
                graceful_restart_helper=dict(type="str", choices=["enabled", "disabled"]),
                keep_alive_interval=dict(type="int"),  # sec
                hold_interval=dict(type="int"),  # sec
                stale_interval=dict(type="int"),  # sec
                max_as_limit=dict(type="int"),
            ),
        ),
        as_path_multipath_relax=(dict(type="str", choices=["enabled", "disabled", ""])),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name", "uuid"], True],
            ["state", "present", ["name", "uuid"], True],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    name = module.params.get("name")
    uuid = module.params.get("uuid")
    description = module.params.get("description")
    bfd_multi_hop_settings = module.params.get("bfd_multi_hop_settings")
    bgp_node_settings = module.params.get("bgp_node_settings")
    as_path_multipath_relax = module.params.get("as_path_multipath_relax")
    state = module.params.get("state")

    mso_template = MSOTemplate(mso, "tenant", template)
    mso_template.validate_template("tenantPolicy")

    path = "/tenantPolicyTemplate/template/l3OutNodePolGroups"

    l3out_node_routing_policy = mso_template.get_l3out_node_routing_policy_object(uuid, name)

    if uuid or name:
        if l3out_node_routing_policy:
            mso.existing = mso.previous = copy.deepcopy(l3out_node_routing_policy.details)  # Query a specific object
    elif l3out_node_routing_policy:
        mso.existing = l3out_node_routing_policy  # Query all objects

    ops = []
    if state == "present":
        if mso.existing:
            proposed_payload = copy.deepcopy(mso.existing)
            update_path = "{0}/{1}".format(path, l3out_node_routing_policy.index)

            if name and proposed_payload.get("name") != name:
                ops.append(dict(op="replace", path="{0}/name".format(update_path), value=name))
                proposed_payload["name"] = name

            if description is not None and proposed_payload.get("description") != description:
                ops.append(dict(op="replace", path="{0}/description".format(update_path), value=description))
                proposed_payload["description"] = description

            if as_path_multipath_relax is not None and as_path_multipath_relax != "":
                if not proposed_payload.get("asPathPol"):
                    proposed_payload["asPathPol"] = dict()
                    ops.append(dict(op="replace", path="{0}/asPathPol".format(update_path), value=dict()))

                if proposed_payload.get("asPathPol").get("asPathMultipathRelax") is not True if as_path_multipath_relax == "enabled" else False:
                    proposed_payload["asPathPol"]["asPathMultipathRelax"] = True if as_path_multipath_relax == "enabled" else False
                    ops.append(
                        dict(
                            op="replace",
                            path="{0}/asPathPol/asPathMultipathRelax".format(update_path),
                            value=True if as_path_multipath_relax == "enabled" else False,
                        )
                    )
            elif as_path_multipath_relax == "":
                proposed_payload.pop("asPathPol", None)
                ops.append(dict(op="remove", path="{0}/asPathPol".format(update_path)))

            if bfd_multi_hop_settings is not None:
                if bfd_multi_hop_settings.get("state") == "disabled" and proposed_payload.get("bfdMultiHopPol"):
                    proposed_payload.pop("bfdMultiHopPol", None)
                    ops.append(dict(op="remove", path="{0}/bfdMultiHopPol".format(update_path)))

                elif bfd_multi_hop_settings.get("state") != "disabled":
                    if not proposed_payload.get("bfdMultiHopPol"):
                        proposed_payload["bfdMultiHopPol"] = dict()
                        ops.append(dict(op="replace", path="{0}/bfdMultiHopPol".format(update_path), value=dict()))

                    if bfd_multi_hop_settings.get("admin_state") is not None and proposed_payload.get("bfdMultiHopPol").get(
                        "adminState"
                    ) != bfd_multi_hop_settings.get("admin_state"):
                        proposed_payload["bfdMultiHopPol"]["adminState"] = bfd_multi_hop_settings.get("admin_state")
                        ops.append(
                            dict(
                                op="replace",
                                path="{0}/bfdMultiHopPol/adminState".format(update_path),
                                value=bfd_multi_hop_settings.get("admin_state"),
                            )
                        )

                    if bfd_multi_hop_settings.get("detection_multiplier") is not None and proposed_payload.get("bfdMultiHopPol").get(
                        "detectionMultiplier"
                    ) != bfd_multi_hop_settings.get("detection_multiplier"):
                        proposed_payload["bfdMultiHopPol"]["detectionMultiplier"] = bfd_multi_hop_settings.get("detection_multiplier")
                        ops.append(
                            dict(
                                op="replace",
                                path="{0}/bfdMultiHopPol/detectionMultiplier".format(update_path),
                                value=bfd_multi_hop_settings.get("detection_multiplier"),
                            )
                        )

                    if bfd_multi_hop_settings.get("min_receive_interval") is not None and proposed_payload.get("bfdMultiHopPol").get(
                        "minRxInterval"
                    ) != bfd_multi_hop_settings.get("min_receive_interval"):
                        proposed_payload["bfdMultiHopPol"]["minRxInterval"] = bfd_multi_hop_settings.get("min_receive_interval")
                        ops.append(
                            dict(
                                op="replace",
                                path="{0}/bfdMultiHopPol/minRxInterval".format(update_path),
                                value=bfd_multi_hop_settings.get("min_receive_interval"),
                            )
                        )

                    if bfd_multi_hop_settings.get("min_transmit_interval") is not None and proposed_payload.get("bfdMultiHopPol").get(
                        "minTxInterval"
                    ) != bfd_multi_hop_settings.get("min_transmit_interval"):
                        proposed_payload["bfdMultiHopPol"]["minTxInterval"] = bfd_multi_hop_settings.get("min_transmit_interval")
                        ops.append(
                            dict(
                                op="replace",
                                path="{0}/bfdMultiHopPol/minTxInterval".format(update_path),
                                value=bfd_multi_hop_settings.get("min_transmit_interval"),
                            )
                        )

            if bgp_node_settings is not None:
                if bgp_node_settings.get("state") == "disabled" and proposed_payload.get("bgpTimerPol"):
                    proposed_payload.pop("bgpTimerPol", None)
                    ops.append(dict(op="remove", path="{0}/bgpTimerPol".format(update_path)))

                elif bgp_node_settings.get("state") != "disabled":
                    if not proposed_payload.get("bgpTimerPol"):
                        proposed_payload["bgpTimerPol"] = dict()
                        ops.append(dict(op="replace", path="{0}/bgpTimerPol".format(update_path), value=dict()))

                    if bgp_node_settings.get("graceful_restart_helper") is not None and (
                        proposed_payload.get("bgpTimerPol").get("gracefulRestartHelper") is not True
                        if bgp_node_settings.get("graceful_restart_helper") == "enabled"
                        else False
                    ):
                        proposed_payload["bgpTimerPol"]["gracefulRestartHelper"] = (
                            True if bgp_node_settings.get("graceful_restart_helper") == "enabled" else False
                        )
                        ops.append(
                            dict(
                                op="replace",
                                path="{0}/bgpTimerPol/gracefulRestartHelper".format(update_path),
                                value=True if bgp_node_settings.get("graceful_restart_helper") == "enabled" else False,
                            )
                        )

                    if bgp_node_settings.get("keep_alive_interval") is not None and proposed_payload.get("bgpTimerPol").get(
                        "keepAliveInterval"
                    ) != bgp_node_settings.get("keep_alive_interval"):
                        proposed_payload["bgpTimerPol"]["keepAliveInterval"] = bgp_node_settings.get("keep_alive_interval")
                        ops.append(
                            dict(
                                op="replace",
                                path="{0}/bgpTimerPol/keepAliveInterval".format(update_path),
                                value=bgp_node_settings.get("keep_alive_interval"),
                            )
                        )

                    if bgp_node_settings.get("hold_interval") is not None and proposed_payload.get("bgpTimerPol").get("holdInterval") != bgp_node_settings.get(
                        "hold_interval"
                    ):
                        proposed_payload["bgpTimerPol"]["holdInterval"] = bgp_node_settings.get("hold_interval")
                        ops.append(
                            dict(
                                op="replace",
                                path="{0}/bgpTimerPol/holdInterval".format(update_path),
                                value=bgp_node_settings.get("hold_interval"),
                            )
                        )

                    if bgp_node_settings.get("stale_interval") is not None and proposed_payload.get("bgpTimerPol").get(
                        "staleInterval"
                    ) != bgp_node_settings.get("stale_interval"):
                        proposed_payload["bgpTimerPol"]["staleInterval"] = bgp_node_settings.get("stale_interval")
                        ops.append(
                            dict(
                                op="replace",
                                path="{0}/bgpTimerPol/staleInterval".format(update_path),
                                value=bgp_node_settings.get("stale_interval"),
                            )
                        )

                    if bgp_node_settings.get("max_as_limit") is not None and proposed_payload.get("bgpTimerPol").get("maxAslimit") != bgp_node_settings.get(
                        "max_as_limit"
                    ):
                        proposed_payload["bgpTimerPol"]["maxAslimit"] = bgp_node_settings.get("max_as_limit")
                        ops.append(
                            dict(
                                op="replace",
                                path="{0}/bgpTimerPol/maxAslimit".format(update_path),
                                value=bgp_node_settings.get("max_as_limit"),
                            )
                        )

            mso.sanitize(proposed_payload)
        else:
            if not (bfd_multi_hop_settings or bgp_node_settings or as_path_multipath_relax):
                mso.fail_json(
                    msg="At least one of the following attributes must be specified when creating L3Out Node Routing Policy:"
                    + " 'bfd_multi_hop_settings', 'bgp_node_settings', or 'as_path_multipath_relax'."
                )

            payload = dict(name=name)
            if description:
                payload["description"] = description

            if as_path_multipath_relax:  # initially ignores None and "" (empty string)
                # enabled = true
                # disabled = false
                # "" = remove the  "BGP Best Path Control" settings from the object
                payload["asPathPol"] = dict(asPathMultipathRelax=True if as_path_multipath_relax == "enabled" else False)

            if bfd_multi_hop_settings is not None:
                bfd_multi_hop_pol = dict()
                if bfd_multi_hop_settings.get("admin_state"):
                    bfd_multi_hop_pol["adminState"] = bfd_multi_hop_settings.get("admin_state")

                if bfd_multi_hop_settings.get("detection_multiplier"):
                    bfd_multi_hop_pol["detectionMultiplier"] = bfd_multi_hop_settings.get("detection_multiplier")

                if bfd_multi_hop_settings.get("min_receive_interval"):
                    bfd_multi_hop_pol["minRxInterval"] = bfd_multi_hop_settings.get("min_receive_interval")

                if bfd_multi_hop_settings.get("min_transmit_interval"):
                    bfd_multi_hop_pol["minTxInterval"] = bfd_multi_hop_settings.get("min_transmit_interval")

                if bfd_multi_hop_pol or bfd_multi_hop_settings.get("state") == "enabled":
                    payload["bfdMultiHopPol"] = bfd_multi_hop_pol

            if bgp_node_settings is not None:
                bgp_timer_pol = dict()
                if bgp_node_settings.get("graceful_restart_helper"):
                    bgp_timer_pol["gracefulRestartHelper"] = True if bgp_node_settings.get("graceful_restart_helper") == "enabled" else False

                if bgp_node_settings.get("keep_alive_interval"):
                    bgp_timer_pol["keepAliveInterval"] = bgp_node_settings.get("keep_alive_interval")

                if bgp_node_settings.get("hold_interval"):
                    bgp_timer_pol["holdInterval"] = bgp_node_settings.get("hold_interval")

                if bgp_node_settings.get("stale_interval"):
                    bgp_timer_pol["staleInterval"] = bgp_node_settings.get("stale_interval")

                if bgp_node_settings.get("max_as_limit"):
                    bgp_timer_pol["maxAslimit"] = bgp_node_settings.get("max_as_limit")

                if bgp_timer_pol or bgp_node_settings.get("state") == "enabled":
                    payload["bgpTimerPol"] = bgp_timer_pol

            ops.append(dict(op="add", path="{0}/-".format(path), value=copy.deepcopy(payload)))

            mso.sanitize(payload)
    elif state == "absent":
        if mso.existing:
            ops.append(dict(op="remove", path="{0}/{1}".format(path, l3out_node_routing_policy.index)))

    if not module.check_mode and ops:
        mso_template.template = mso.request(mso_template.template_path, method="PATCH", data=ops)
        l3out_node_routing_policy = mso_template.get_l3out_node_routing_policy_object(uuid, name)
        if l3out_node_routing_policy:
            mso.existing = l3out_node_routing_policy.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: hosttech_dns_zone_info

short_description: Retrieve zone information in Hosttech DNS service

version_added: 0.2.0

description:
  - Retrieves zone information in Hosttech DNS service.
extends_documentation_fragment:
  - community.dns._hosttech
  - community.dns._hosttech.zone_id_type
  - community.dns._module_zone_info
  - community.dns._attributes
  - community.dns._attributes.actiongroup_hosttech
  - community.dns._attributes.info_module
  - community.dns._attributes.idempotent_not_modify_state

attributes:
  action_group:
    version_added: 2.4.0

author:
  - Felix Fontein (@felixfontein)
"""

EXAMPLES = r"""
- name: Retrieve details for foo.com zone
  community.dns.hosttech_dns_zone_info:
    zone_name: foo.com
    hosttech_username: foo
    hosttech_password: bar
  register: rec

- name: Retrieve details for zone 23
  community.dns.hosttech_dns_zone_info:
    zone_id: 23
    hosttech_token: access_token
"""

RETURN = r"""
zone_name:
  description: The name of the zone.
  type: int
  returned: success
  sample: example.com

zone_id:
  description: The ID of the zone.
  type: int
  returned: success
  sample: 23

zone_info:
  description:
    - Extra information returned by the API.
  type: dict
  returned: success
  version_added: 2.0.0
  sample:
    dnssec: true
    dnssec_email: test@example.com
    ds_records: []
    email: test@example.com
    ttl: 3600
  contains:
    dnssec:
      description:
        - Whether DNSSEC is enabled for the zone or not.
      type: bool
      returned: When O(hosttech_token) has been specified.
    dnssec_email:
      description:
        - The email address contacted when the DNSSEC key is changed.
        - Is V(none) if DNSSEC is not enabled.
      type: str
      returned: When O(hosttech_token) has been specified.
    ds_records:
      description:
        - The DS records.
        - See L(Section 5 of RFC 4034,https://datatracker.ietf.org/doc/html/rfc4034#section-5) and
          L(Section 2.1 of RFC 4034,https://datatracker.ietf.org/doc/html/rfc4034#section-2.1) for details.
        - Is V(none) if DNSSEC is not enabled.
      type: list
      elements: dict
      returned: When O(hosttech_token) has been specified.
      contains:
        algorithm:
          description:
            - This value is the algorithm number of the DNSKEY RR referred to by the DS record.
            - A list of values can be found in L(Appendix A.1 of RFC 4034,https://datatracker.ietf.org/doc/html/rfc4034#appendix-A.1).
          type: int
          sample: 8
        digest:
          description:
            - A digest of the DNSKEY RR record this DS record refers to.
          type: str
          sample: 012356789ABCDEF0123456789ABCDEF012345678
        digest_type:
          description:
            - This value identifies the algorithm used to construct the digest.
            - A list of values can be found in L(Appendix A.2 of RFC 4034,https://datatracker.ietf.org/doc/html/rfc4034#appendix-A.2).
          type: int
          sample: 1
        flags:
          description:
            - The Zone Key flag. See L(Section 2.1.1 of RFC 4034,https://datatracker.ietf.org/doc/html/rfc4034#section-2.1.1)
              for details.
          type: int
          sample: 257
        key_tag:
          description:
            - The Key Tag field lists the key tag of the DNSKEY RR referred to by the DS record.
          type: int
          sample: 12345
        protocol:
          description:
            - Must be 3 according to RFC 4034.
          type: int
          sample: 3
        public_key:
          description:
            - The public key material.
          type: str
          sample: >-
            MuhdzsQdqEGShwjtJDKZZjdKqUSGluFzTTinpuEeIRzLLcgkwgAPKWFa
            eQntNlmcNDeCziGwpdvhJnvKXEMbFcZwsaDIJuWqERxAQNGABWfPlCLh
            HQPnbpRPNKipSdBaUhuOubvFvjBpFAwiwSAapRDVsAgKvjXucfXpFfYb
            pCundbAXBWhbpHVbqgmGoixXzFSwUsGVYLPpBCiDlLJwzjRKYYaoVYge
            kMtKFYUVnWIKbectWkDFdVqXwkKigCUDiuTTJxOBRJRNzGiDNMWBjYSm
            bBCAHMaMYaghLbYTwyKXltdHTHwBwtswGNfpnEdSpKFzZJonBZArQfHD
            lfceKgmKwEF=
    email:
      description:
        - The zone's DNS contact mail in the SOA record.
      type: str
    ttl:
      description:
        - The zone's TTL.
      type: int
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.dns.plugins.module_utils._argspec import (
    ModuleOptionProvider,
)
from ansible_collections.community.dns.plugins.module_utils._hosttech.api import (
    create_hosttech_api,
    create_hosttech_argument_spec,
    create_hosttech_provider_information,
)
from ansible_collections.community.dns.plugins.module_utils._http import (
    ModuleHTTPHelper,
)
from ansible_collections.community.dns.plugins.module_utils._module.zone_info import (
    create_module_argument_spec,
    run_module,
)


def main() -> None:
    provider_information = create_hosttech_provider_information()
    argument_spec = create_hosttech_argument_spec()
    argument_spec.merge(
        create_module_argument_spec(provider_information=provider_information)
    )
    module = AnsibleModule(supports_check_mode=True, **argument_spec.to_kwargs())
    run_module(
        module,
        lambda: create_hosttech_api(
            ModuleOptionProvider(module), ModuleHTTPHelper(module)
        ),
        provider_information=provider_information,
    )


if __name__ == "__main__":
    main()



# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# (c) 2016 Red Hat Inc.
# (c) 2019 Cisco Systems Inc.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type
import traceback

from ansible.module_utils.basic import missing_required_lib

UCSMSDK_IMP_ERR = None
try:
    # import done here to provide common import check for all modules
    # pylint: disable=unused-import
    import ucsmsdk
    HAS_UCSMSDK = True
except Exception:
    UCSMSDK_IMP_ERR = traceback.format_exc()
    HAS_UCSMSDK = False

ucs_argument_spec = dict(
    hostname=dict(type='str', required=True),
    username=dict(type='str', default='admin'),
    password=dict(type='str', required=True, no_log=True),
    port=dict(type='int', default=None),
    use_ssl=dict(type='bool', default=True),
    use_proxy=dict(type='bool', default=True),
    proxy=dict(type='str', default=None),
)


class UCSModule():

    def __init__(self, module):
        self.module = module
        self.result = {}
        if not HAS_UCSMSDK:
            self.module.fail_json(msg=missing_required_lib('ucsmsdk'), exception=UCSMSDK_IMP_ERR)
        self.login()

    def __del__(self):
        self.logout()

    def login(self):
        from ucsmsdk.ucshandle import UcsHandle

        # use_proxy=yes (default) and proxy=None (default) should be using the system defined proxy
        # use_proxy=yes (default) and proxy=value should use the provided proxy
        # use_proxy=no (user) should not be using a proxy
        if self.module.params['use_proxy']:
            proxy = self.module.params['proxy']
        else:
            # force no proxy to be used.  Note that proxy=None in UcsHandle will
            # use the system proxy so we must set to something else
            proxy = {}

        try:
            handle = UcsHandle(
                ip=self.module.params['hostname'],
                username=self.module.params['username'],
                password=self.module.params['password'],
                port=self.module.params['port'],
                secure=self.module.params['use_ssl'],
                proxy=proxy
            )
            handle.login()
        except Exception as e:
            self.result['msg'] = str(e)
            self.module.fail_json(**self.result)
        self.login_handle = handle

    def logout(self):
        if hasattr(self, 'login_handle'):
            self.login_handle.logout()
            return True
        return False



#!/usr/bin/python
#
# Copyright (c) Ansible Project
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: jenkins_job_info
short_description: Get information about Jenkins jobs
description:
  - This module can be used to query information about which Jenkins jobs which already exists.
requirements:
  - "python-jenkins >= 0.4.12"
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.info_module
options:
  name:
    type: str
    description:
      - Exact name of the Jenkins job to fetch information about.
  glob:
    type: str
    description:
      - A shell glob of Jenkins job names to fetch information about.
  color:
    type: str
    description:
      - Only fetch jobs with the given status color.
  password:
    type: str
    description:
      - Password to authenticate with the Jenkins server.
      - This is mutually exclusive with O(token).
  token:
    type: str
    description:
      - API token used to authenticate with the Jenkins server.
      - This is mutually exclusive with O(password).
  url:
    type: str
    description:
      - URL where the Jenkins server is accessible.
    default: http://localhost:8080
  user:
    type: str
    description:
      - User to authenticate with the Jenkins server.
  validate_certs:
    description:
      - If set to V(false), the SSL certificates are not validated.
      - This should only set to V(false) used on personally controlled sites using self-signed certificates.
    default: true
    type: bool
author:
  - "Chris St. Pierre (@stpierre)"
"""

EXAMPLES = r"""
# Get all Jenkins jobs anonymously
- community.general.jenkins_job_info:
    user: admin
  register: my_jenkins_job_info

# Get all Jenkins jobs using basic auth
- community.general.jenkins_job_info:
    user: admin
    password: hunter2
  register: my_jenkins_job_info

# Get all Jenkins jobs using the token
- community.general.jenkins_job_info:
    user: admin
    token: abcdefghijklmnop
  register: my_jenkins_job_info

# Get info about a single job using basic auth
- community.general.jenkins_job_info:
    name: some-job-name
    user: admin
    password: hunter2
  register: my_jenkins_job_info

# Get info about a single job in a folder using basic auth
- community.general.jenkins_job_info:
    name: some-folder-name/some-job-name
    user: admin
    password: hunter2
  register: my_jenkins_job_info

# Get info about jobs matching a shell glob using basic auth
- community.general.jenkins_job_info:
    glob: some-job-*
    user: admin
    password: hunter2
  register: my_jenkins_job_info

# Get info about all failing jobs using basic auth
- community.general.jenkins_job_info:
    color: red
    user: admin
    password: hunter2
  register: my_jenkins_job_info

# Get info about passing jobs matching a shell glob using basic auth
- community.general.jenkins_job_info:
    name: some-job-*
    color: blue
    user: admin
    password: hunter2
  register: my_jenkins_job_info

- name: Get the info from custom URL with token and validate_certs=False
  community.general.jenkins_job_info:
    user: admin
    token: 126df5c60d66c66e3b75b11104a16a8a
    url: https://jenkins.example.com
  register: my_jenkins_job_info
"""

RETURN = r"""
jobs:
  description: All jobs found matching the specified criteria.
  returned: success
  type: list
  sample:
    [
      {
        "name": "test-job",
        "fullname": "test-folder/test-job",
        "url": "http://localhost:8080/job/test-job/",
        "color": "blue"
      }
    ]
"""

import fnmatch
import ssl
import traceback

JENKINS_IMP_ERR = None
try:
    import jenkins

    HAS_JENKINS = True
except ImportError:
    JENKINS_IMP_ERR = traceback.format_exc()
    HAS_JENKINS = False

from ansible.module_utils.basic import AnsibleModule, missing_required_lib


def get_jenkins_connection(module):
    url = module.params["url"]
    username = module.params.get("user")
    password = module.params.get("password")
    token = module.params.get("token")

    validate_certs = module.params.get("validate_certs")
    if not validate_certs:
        ssl._create_default_https_context = ssl._create_unverified_context

    if username and (password or token):
        return jenkins.Jenkins(url, username, password or token)
    elif username:
        return jenkins.Jenkins(url, username)
    else:
        return jenkins.Jenkins(url)


def test_dependencies(module):
    if not HAS_JENKINS:
        module.fail_json(
            msg=missing_required_lib(
                "python-jenkins", url="https://python-jenkins.readthedocs.io/en/latest/install.html"
            ),
            exception=JENKINS_IMP_ERR,
        )


def get_jobs(module):
    jenkins_conn = get_jenkins_connection(module)
    jobs = []
    if module.params.get("name"):
        try:
            job_info = jenkins_conn.get_job_info(module.params.get("name"))
        except jenkins.NotFoundException:
            pass
        else:
            jobs.append(
                {
                    "name": job_info["name"],
                    "fullname": job_info["fullName"],
                    "url": job_info["url"],
                    "color": job_info["color"],
                }
            )

    else:
        all_jobs = jenkins_conn.get_all_jobs()
        if module.params.get("glob"):
            jobs.extend(j for j in all_jobs if fnmatch.fnmatch(j["fullname"], module.params.get("glob")))
        else:
            jobs = all_jobs
        # python-jenkins includes the internal Jenkins class used for each job
        # in its return value; we strip that out because the leading underscore
        # (and the fact that it is not documented in the python-jenkins docs)
        # indicates that it is not part of the dependable public interface.
        for job in jobs:
            if "_class" in job:
                del job["_class"]

    if module.params.get("color"):
        jobs = [j for j in jobs if j["color"] == module.params.get("color")]

    return jobs


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(type="str"),
            glob=dict(type="str"),
            color=dict(type="str"),
            password=dict(type="str", no_log=True),
            token=dict(type="str", no_log=True),
            url=dict(type="str", default="http://localhost:8080"),
            user=dict(type="str"),
            validate_certs=dict(type="bool", default=True),
        ),
        mutually_exclusive=[
            ["password", "token"],
            ["name", "glob"],
        ],
        supports_check_mode=True,
    )

    test_dependencies(module)
    jobs = list()

    try:
        jobs = get_jobs(module)
    except jenkins.JenkinsException as err:
        module.fail_json(msg=f"Unable to connect to Jenkins server, {err}", exception=traceback.format_exc())

    module.exit_json(changed=False, jobs=jobs)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for cisco.ios_route_maps
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: ios_route_maps
short_description: Resource module to configure route maps.
description:
  - This module configures and manages the attributes of Route maps on Cisco IOS.
version_added: 2.1.0
author: Sumit Jaiswal (@justjais)
notes:
  - Tested against Cisco IOSXE Version 17.3 on CML.
  - This module works with connection C(network_cli).
    See U(https://docs.ansible.com/ansible/latest/network/user_guide/platform_ios.html)
options:
  config:
    description: A list of configurations for Route maps.
    type: list
    elements: dict
    suboptions:
      route_map:
        description: Route map tag/name
        type: str
      entries:
        description: A list of configurations entries for Route maps.
        type: list
        elements: dict
        suboptions:
          sequence:
            description:
              - Sequence to insert to/delete from existing route-map entry
              - Please refer vendor documentation for valid values
            type: int
          action:
            description: Route map set operations
            type: str
            choices: ["deny", "permit"]
          continue_entry:
            description: Continue on a different entry within the route-map
            type: dict
            suboptions:
              set:
                description: Set continue
                type: bool
              entry_sequence:
                description:
                  - Route-map entry sequence number
                  - Please refer vendor documentation for valid values
                type: int
          description:
            description: Route-map comment
            type: str
          match:
            description: Match values from routing table
            type: dict
            suboptions:
              additional_paths:
                description:
                  - BGP Add-Path match policie
                  - BGP Add-Path advertise-set policy
                type: dict
                suboptions:
                  all:
                    description: BGP Add-Path advertise all paths
                    type: bool
                  best:
                    description: BGP Add-Path advertise best n paths (1-3)
                    type: int
                  best_range:
                    description: BGP Add-Path advertise best paths (range m to n)
                    type: dict
                    suboptions:
                      lower_limit:
                        description: BGP Add-Path best paths to advertise (lower limit) (1-3)
                        type: int
                      upper_limit:
                        description: BGP Add-Path best paths to advertise (upper limit) (1-3)
                        type: int
                  group_best:
                    description: BGP Add-Path advertise group-best path
                    type: bool
              as_path:
                description: Match BGP AS path list
                type: dict
                suboptions:
                  set:
                    description: Set AS path list
                    type: bool
                  acls:
                    description:
                      - AS path access-list
                      - Please refer vendor documentation for valid values
                    type: list
                    elements: int
              clns:
                description: CLNS information
                type: dict
                suboptions:
                  address:
                    description: Match address of route or match packet
                    type: str
                  next_hop:
                    description: Match next-hop address of route
                    type: str
                  route_source:
                    description: Match advertising source address of route
                    type: str
              community:
                description: Match BGP community list
                type: dict
                suboptions:
                  name:
                    description:
                      - Community-list number/Community-list name
                      - Please refer vendor documentation for valid values
                    type: list
                    elements: str
                  exact_match:
                    description: Do exact matching of communities
                    type: bool
              extcommunity:
                description:
                  - Match BGP/VPN extended community list
                  - Extended community-list number
                  - Please refer vendor documentation for valid values
                type: list
                elements: str
              interfaces:
                description: Match first hop interface of route
                type: list
                elements: str
              ip:
                description: IP specific information
                type: dict
                suboptions:
                  address:
                    description: Match address of route or match packet
                    type: dict
                    suboptions:
                      acls: &acls
                        description:
                          - Match entries of acl
                          - IP acl name/number
                          - Please refer vendor documentation for valid values
                        type: list
                        elements: str
                      prefix_lists: &prefix_lists
                        description:
                          - Match entries of prefix-lists
                          - IP prefix-list name
                        type: list
                        elements: str
                  flowspec:
                    description: Match src/dest prefix component of flowspec prefix
                    type: dict
                    suboptions:
                      dest_pfx:
                        description: Match dest prefix component of flowspec prefix
                        type: bool
                      src_pfx:
                        description: Match source prefix component of flowspec prefix
                        type: bool
                      acls: *acls
                      prefix_lists: *prefix_lists
                  next_hop:
                    description: Match next-hop address of route
                    type: dict
                    suboptions:
                      set:
                        description: Set next-hop address
                        type: bool
                      acls: *acls
                      prefix_lists: *prefix_lists
                  redistribution_source:
                    description: route redistribution source (EIGRP only)
                    type: dict
                    suboptions:
                      set:
                        description: Set redistribution-source
                        type: bool
                      acls: *acls
                      prefix_lists: *prefix_lists
                  route_source:
                    description: Match advertising source address of route
                    type: dict
                    suboptions:
                      set:
                        description: Set redistribution-source
                        type: bool
                      redistribution_source:
                        description: route redistribution source (EIGRP only)
                        type: bool
                      acls: *acls
                      prefix_lists: *prefix_lists
              ipv6:
                description: IPv6 specific information
                type: dict
                suboptions:
                  address:
                    type: dict
                    description: Match address of route or match packet
                    suboptions:
                      acl:
                        description: IPv6 access-list name
                        type: str
                      prefix_list:
                        description: IPv6 prefix-list name
                        type: str
                  flowspec:
                    description: Match next-hop address of route
                    type: dict
                    suboptions:
                      dest_pfx:
                        description: Match dest prefix component of flowspec prefix
                        type: bool
                      src_pfx:
                        description: Match source prefix component of flowspec prefix
                        type: bool
                      acl:
                        description: IPv6 access-list name
                        type: str
                      prefix_list:
                        description: IPv6 prefix-list name
                        type: str
                  next_hop:
                    description: Match next-hop address of route
                    type: dict
                    suboptions:
                      acl:
                        description: IPv6 access-list name
                        type: str
                      prefix_list:
                        description: IPv6 prefix-list name
                        type: str
                  route_source:
                    description: Match advertising source address of route
                    type: dict
                    suboptions:
                      acl:
                        description: IPv6 access-list name
                        type: str
                      prefix_list:
                        description: IPv6 prefix-list name
                        type: str
              length:
                description: Packet length
                type: dict
                suboptions:
                  minimum:
                    description:
                      - Minimum packet length
                      - Please refer vendor documentation for valid values
                    type: int
                  maximum:
                    description:
                      - Maximum packet length
                      - Please refer vendor documentation for valid values
                    type: int
              local_preference:
                description: Local preference for route
                type: dict
                suboptions:
                  set:
                    description: Set the Local preference for route
                    type: bool
                  value:
                    description:
                      - Local preference value
                      - Please refer vendor documentation for valid values
                    type: list
                    elements: str
              mdt_group:
                description: Match routes corresponding to MDT group
                type: dict
                suboptions:
                  set:
                    description: Set and Match routes corresponding to MDT group
                    type: bool
                  acls:
                    description:
                      - IP access-list number/IP standard access-list name
                      - Please refer vendor documentation for valid values
                    type: list
                    elements: str
              metric:
                description: Match metric of route
                type: dict
                suboptions:
                  value:
                    description:
                      - Metric value
                      - Please refer vendor documentation for valid values
                    type: int
                  external:
                    description: Match route using external protocol metric
                    type: bool
                  deviation:
                    description: Deviation option to match metric in a range
                    type: bool
                  deviation_value:
                    description:
                      - deviation value, 500 +- 10 creates the range 490 - 510
                      - Please refer vendor documentation for valid values
                    type: int
              mpls_label:
                description: Match routes which have MPLS labels
                type: bool
              policy_lists:
                description: Match IP policy list
                type: list
                elements: str
              route_type:
                description: Match route-type of route
                type: dict
                suboptions:
                  external:
                    description: external route (BGP, EIGRP and OSPF type 1/2)
                    type: dict
                    suboptions:
                      set:
                        description: Set external route
                        type: bool
                      type_1:
                        description: OSPF external type 1 route
                        type: bool
                      type_2:
                        description: OSPF external type 2 route
                        type: bool
                  internal:
                    description: internal route (including OSPF intra/inter area)
                    type: bool
                  level_1:
                    description: IS-IS level-1 route
                    type: bool
                  level_2:
                    description: IS-IS level-2 route
                    type: bool
                  local:
                    description: locally generated route
                    type: bool
                  nssa_external:
                    description: nssa-external route (OSPF type 1/2)
                    type: dict
                    suboptions:
                      set:
                        description: Set nssa-external route
                        type: bool
                      type_1:
                        description: OSPF external type 1 route
                        type: bool
                      type_2:
                        description: OSPF external type 2 route
                        type: bool
              rpki:
                description: Match RPKI state of route
                type: dict
                suboptions:
                  invalid:
                    description: RPKI Invalid State
                    type: bool
                  not_found:
                    description: RPKI Not Found State
                    type: bool
                  valid:
                    description: RPKI Valid State
                    type: bool
              security_group:
                description: Security Group
                type: dict
                suboptions:
                  source:
                    description:
                      - Source Security Group, source Security tag
                      - Please refer vendor documentation for valid values
                    type: list
                    elements: int
                  destination:
                    description:
                      - Destination Security Group, destination Security tag
                      - Please refer vendor documentation for valid values
                    type: list
                    elements: int
              source_protocol:
                description: Match source-protocol of route
                type: dict
                suboptions:
                  bgp:
                    description:
                      - Border Gateway Protocol (BGP)
                      - Autonomous system number
                      - Please refer vendor documentation for valid values
                    type: str
                  connected:
                    description: Connected
                    type: bool
                  eigrp:
                    description:
                      - Enhanced Interior Gateway Routing Protocol (EIGRP)
                      - Autonomous system number
                      - Please refer vendor documentation for valid values
                    type: int
                  isis:
                    description: ISO IS-IS
                    type: bool
                  lisp:
                    description: Locator ID Separation Protocol (LISP)
                    type: bool
                  mobile:
                    description: Mobile routes
                    type: bool
                  ospf:
                    description:
                      - Open Shortest Path First (OSPF) Process ID
                      - Please refer vendor documentation for valid values
                    type: int
                  ospfv3:
                    description:
                      - OSPFv3 Process ID
                      - Please refer vendor documentation for valid values
                    type: int
                  rip:
                    description: Routing Information Protocol (RIP)
                    type: bool
                  static:
                    description: Static routes
                    type: bool
              tag:
                description: Match tag of route
                type: dict
                suboptions:
                  value:
                    description: Tag value/Tag in Dotted Decimal eg, 10.10.10.10
                    type: list
                    elements: str
                  tag_list:
                    description: Route Tag List/Tag list name
                    type: list
                    elements: str
              track:
                description: tracking object
                type: int
          set:
            description: Match source-protocol of route
            type: dict
            suboptions:
              aigp_metric:
                description: accumulated metric value
                type: dict
                suboptions:
                  value:
                    description: manual value
                    type: int
                  igp_metric:
                    description: metric value from rib
                    type: bool
              as_path:
                description: Prepend string for a BGP AS-path attribute
                type: dict
                suboptions:
                  prepend:
                    description: Prepend to the as-path
                    type: dict
                    suboptions:
                      as_number:
                        description:
                          - AS number
                          - Please refer vendor documentation for valid values
                        type: list
                        elements: str
                      last_as:
                        description:
                          - Prepend last AS to the as-path
                          - Number of last-AS prepends
                          - Please refer vendor documentation for valid values
                        type: int
                  tag:
                    description: Set the tag as an AS-path attribute
                    type: bool
              automatic_tag:
                description: Automatically compute TAG value
                type: bool
              clns:
                description:
                  - OSI summary address
                  - Next hop address
                  - CLNS summary prefix
                type: str
              comm_list:
                description:
                  - set BGP community list (for deletion)
                  - Community-list name/number
                  - Delete matching communities
                type: str
              community:
                description: BGP community attribute
                type: dict
                suboptions:
                  number:
                    description:
                      - community number
                      - community number in aa:nn format
                      - Please refer vendor documentation for valid values
                    type: str
                  additive:
                    description: Add to the existing community
                    type: bool
                  gshut:
                    description: Graceful Shutdown (well-known community)
                    type: bool
                  internet:
                    description: Internet (well-known community)
                    type: bool
                  local_as:
                    description: Do not send outside local AS (well-known community)
                    type: bool
                  no_advertise:
                    description: Do not advertise to any peer (well-known community)
                    type: bool
                  no_export:
                    description: Do not export to next AS (well-known community)
                    type: bool
                  none:
                    description: No community attribute
                    type: bool
              dampening:
                description: Set BGP route flap dampening parameters
                type: dict
                suboptions:
                  penalty_half_time:
                    description:
                      - half-life time for the penalty
                      - Please refer vendor documentation for valid values
                    type: int
                  reuse_route_val:
                    description:
                      - Penalty to start reusing a route
                      - Please refer vendor documentation for valid values
                    type: int
                  suppress_route_val:
                    description:
                      - Penalty to start suppressing a route
                      - Please refer vendor documentation for valid values
                    type: int
                  max_suppress:
                    description:
                      - Maximum duration to suppress a stable route
                      - Please refer vendor documentation for valid values
                    type: int
              default:
                description:
                  - Set default information
                  - Default output interface
                type: str
              extcomm_list:
                description:
                  - Set BGP/VPN extended community list (for deletion)
                  - Extended community-list number/name
                  - Delete matching extended communities
                type: str
              extcommunity:
                description: BGP extended community attribute
                type: dict
                suboptions:
                  cost:
                    description: Cost extended community
                    type: dict
                    suboptions:
                      id:
                        description:
                          - Community ID
                          - Please refer vendor documentation for valid values
                        type: str
                      cost_value:
                        description:
                          - Cost Value (No-preference Cost = 2147483647)
                          - Please refer vendor documentation for valid values
                        type: int
                      igp:
                        description: Compare following IGP cost comparison
                        type: bool
                      pre_bestpath:
                        description: Compare before all other steps in bestpath calculation
                        type: bool
                  rt:
                    description: Route Target extended community
                    type: dict
                    suboptions:
                      address:
                        description: VPN extended community
                        type: str
                      range:
                        description: Specify a range of extended community
                        type: dict
                        suboptions:
                          lower_limit:
                            description: VPN extended community
                            type: str
                          upper_limit:
                            description: VPN extended community
                            type: str
                      additive:
                        description: Add to the existing extcommunity
                        type: bool
                  soo:
                    description: Site-of-Origin extended community
                    type: str
                  vpn_distinguisher:
                    description: VPN Distinguisher
                    type: dict
                    suboptions:
                      address:
                        description: VPN extended community
                        type: str
                      range:
                        description: Specify a range of extended community
                        type: dict
                        suboptions:
                          lower_limit:
                            description: VPN extended community
                            type: str
                          upper_limit:
                            description: VPN extended community
                            type: str
                      additive:
                        description: Add to the existing extcommunity
                        type: bool
              global_route:
                description: Set to global routing table
                type: bool
              interfaces:
                description: Output interface
                type: list
                elements: str
              ip:
                description: IP specific information
                type: dict
                suboptions:
                  address:
                    description:
                      - Specify IP address
                      - Prefix-list name to set ip address
                    type: str
                  df:
                    description: Set DF bit
                    choices: [0, 1]
                    type: int
                  global_route:
                    description: global routing table
                    type: dict
                    suboptions:
                      address:
                        description: IP address of next hop
                        type: str
                      verify_availability:
                        description: Verify if nexthop is reachable
                        type: dict
                        suboptions:
                          address:
                            description: IP address of next hop
                            type: str
                          sequence:
                            description:
                              - Sequence to insert into next-hop list
                              - Please refer vendor documentation for valid values
                            type: int
                          track:
                            description:
                              - Set the next hop depending on the state of a tracked object
                              - tracked object number
                              - Please refer vendor documentation for valid values
                            type: int
                  next_hop:
                    description: Next hop address
                    type: dict
                    suboptions:
                      address:
                        description: IP address of next hop
                        type: str
                      dynamic:
                        description:
                          - application dynamically sets next hop
                          - DHCP learned next hop
                        type: bool
                      encapsulate:
                        description:
                          - Encapsulation profile for VPN nexthop
                          - L3VPN
                          - Encapsulation profile name
                        type: str
                      peer_address:
                        description: Use peer address (for BGP only)
                        type: bool
                      recursive:
                        description: Recursive next-hop
                        type: dict
                        suboptions:
                          global_route:
                            description: global routing table
                            type: bool
                          vrf:
                            description: VRF
                            type: str
                          address:
                            description: IP address of recursive next hop
                            type: str
                      self:
                        description: Use self address (for BGP only)
                        type: bool
                      verify_availability:
                        description: Verify if nexthop is reachable
                        type: dict
                        suboptions:
                          set:
                            description: Set and Verify if nexthop is reachable
                            type: bool
                          address:
                            description: IP address of next hop
                            type: str
                          sequence:
                            description:
                              - Sequence to insert into next-hop list
                              - Please refer vendor documentation for valid values
                            type: int
                          track:
                            description:
                              - Set the next hop depending on the state of a tracked object
                              - tracked object number
                              - Please refer vendor documentation for valid values
                            type: int
                  precedence:
                    description: Set precedence field
                    type: dict
                    suboptions:
                      set:
                        description: Just set precedence field
                        type: bool
                      critical:
                        description: Set critical precedence (5)
                        type: bool
                      flash:
                        description: Set flash precedence (3)
                        type: bool
                      flash_override:
                        description: Set flash override precedence (4)
                        type: bool
                      immediate:
                        description: Set immediate precedence (2)
                        type: bool
                      internet:
                        description: Set internetwork control precedence (6)
                        type: bool
                      network:
                        description: Set network control precedence (7)
                        type: bool
                      priority:
                        description: Set priority precedence (1)
                        type: bool
                      routine:
                        description: Set routine precedence (0)
                        type: bool
                  qos_group:
                    description:
                      - Set QOS Group ID
                      - Please refer vendor documentation for valid values
                    type: int
                  tos:
                    description: Set type of service field
                    type: dict
                    suboptions:
                      set:
                        description: Just set type of service field
                        type: bool
                      max_reliability:
                        description: Set max reliable TOS (2)
                        type: bool
                      max_throughput:
                        description: Set max throughput TOS (4)
                        type: bool
                      min_delay:
                        description: Set min delay TOS (8)
                        type: bool
                      min_monetary_cost:
                        description: Set min monetary cost TOS (1)
                        type: bool
                      normal:
                        description: Set normal TOS (0)
                        type: bool
                  vrf:
                    description: VRF
                    type: dict
                    suboptions:
                      name:
                        description: VRF name
                        type: str
                      address:
                        description: IP address of next hop
                        type: str
                      verify_availability:
                        description: Verify if nexthop is reachable
                        type: dict
                        suboptions:
                          set:
                            description: Set and Verify if nexthop is reachable
                            type: bool
                          address:
                            description: IP address of next hop
                            type: str
                          sequence:
                            description:
                              - Sequence to insert into next-hop list
                              - Please refer vendor documentation for valid values
                            type: int
                          track:
                            description:
                              - Set the next hop depending on the state of a tracked object
                              - tracked object number
                              - Please refer vendor documentation for valid values
                            type: int
              ipv6:
                description: IPv6 specific information
                type: dict
                suboptions:
                  address:
                    description:
                      - IPv6 address
                      - IPv6 prefix-list
                    type: str
                  default:
                    description: Set default information
                    type: bool
                  global_route:
                    description: global routing table
                    type: dict
                    suboptions:
                      address:
                        description: Next hop address (X:X:X:X::X)
                        type: str
                      verify_availability:
                        description: Verify if nexthop is reachable
                        type: dict
                        suboptions:
                          address:
                            description: Next hop address (X:X:X:X::X)
                            type: str
                          sequence:
                            description:
                              - Sequence to insert into next-hop list
                              - Please refer vendor documentation for valid values
                            type: int
                          track:
                            description:
                              - Set the next hop depending on the state of a tracked object
                              - tracked object number
                              - Please refer vendor documentation for valid values
                            type: int
                  next_hop:
                    description: IPv6 Next hop
                    type: dict
                    suboptions:
                      address:
                        description: Next hop address (X:X:X:X::X)
                        type: str
                      encapsulate:
                        description:
                          - Encapsulation profile for VPN nexthop
                          - L3VPN
                          - Encapsulation profile name
                        type: str
                      peer_address:
                        description: Use peer address (for BGP only)
                        type: bool
                      recursive:
                        description:
                          - Recursive next-hop
                          - IPv6 address of recursive next-hop
                        type: str
                  precedence:
                    description:
                      - Set precedence field
                      - Precedence value
                      - Please refer vendor documentation for valid values
                    type: int
                  vrf:
                    description: VRF name
                    type: dict
                    suboptions:
                      name:
                        description: VRF name
                        type: str
                      verify_availability:
                        description: Verify if nexthop is reachable
                        type: dict
                        suboptions:
                          address:
                            description: IPv6 address of next hop
                            type: str
                          sequence:
                            description:
                              - Sequence to insert into next-hop list
                              - Please refer vendor documentation for valid values
                            type: int
                          track:
                            description:
                              - Set the next hop depending on the state of a tracked object
                              - tracked object number
                              - Please refer vendor documentation for valid values
                            type: int
              level:
                description: Where to import route
                type: dict
                suboptions:
                  level_1:
                    description: Import into a level-1 area
                    type: bool
                  level_1_2:
                    description: Import into level-1 and level-2
                    type: bool
                  level_2:
                    description: Import into level-2 sub-domain
                    type: bool
                  nssa_only:
                    description: Import only into OSPF NSSA areas and don't propagate
                    type: bool
              lisp:
                description:
                  - Locator ID Separation Protocol specific information
                  - Specify a locator-set to use in LISP route-import
                  - The name of the locator set
                type: str
              local_preference:
                description:
                  - BGP local preference path attribute
                  - Please refer vendor documentation for valid values
                type: int
              metric:
                description: Metric value for destination routing protocol
                type: dict
                suboptions:
                  deviation:
                    description: Add or subtract metric
                    choices: ["plus", "minus"]
                    type: str
                  metric_value:
                    description:
                      - Metric value or Bandwidth in Kbits per second
                      - Please refer vendor documentation for valid values
                    type: int
                  eigrp_delay:
                    description:
                      - EIGRP delay metric, in 10 microsecond units
                      - Please refer vendor documentation for valid values
                    type: int
                  metric_reliability:
                    description:
                      - EIGRP reliability metric where 255 is 100 reliable
                      - Please refer vendor documentation for valid values
                    type: int
                  metric_bandwidth:
                    description:
                      - EIGRP Effective bandwidth metric (Loading) where 255 is 100 loaded
                      - Please refer vendor documentation for valid values
                    type: int
                  mtu:
                    description:
                      - EIGRP MTU of the path
                      - Please refer vendor documentation for valid values
                    type: int
              metric_type:
                description: Type of metric for destination routing protocol
                type: dict
                suboptions:
                  external:
                    description: IS-IS external metric
                    type: bool
                  internal:
                    description: IS-IS internal metric or Use IGP metric as the MED for BGP
                    type: bool
                  type_1:
                    description: OSPF external type 1 metric
                    type: bool
                  type_2:
                    description: OSPF external type 2 metric
                    type: bool
              mpls_label:
                description: Set MPLS label for prefix
                type: bool
              origin:
                description: BGP origin code
                type: dict
                suboptions:
                  igp:
                    description: local IGP
                    type: bool
                  incomplete:
                    description: unknown heritage
                    type: bool
              tag:
                description:
                  - Tag value for destination routing protocol
                  - Tag value A.B.C.D(dotted decimal format)/Tag value
                type: str
              traffic_index:
                description:
                  - BGP traffic classification number for accounting
                  - Please refer vendor documentation for valid values
                type: int
              vrf:
                description:
                  - Define VRF name
                  - VPN Routing/Forwarding instance name
                type: str
              weight:
                description:
                  - BGP weight for routing table
                  - Please refer vendor documentation for valid values
                type: int
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS
        device by executing the command B(sh running-config | section ^route-map).
      - The state I(parsed) reads the configuration from C(running_config)
        option and transforms it into Ansible structured data as per the
        resource module's argspec and the value is then returned in the
        I(parsed) key within the result.
    type: str
  state:
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - gathered
      - parsed
      - rendered
    default: merged
    description:
      - The state the configuration should be left in
      - The states I(rendered), I(gathered) and I(parsed) does not perform any change
        on the device.
      - The state I(rendered) will transform the configuration in C(config) option to
        platform specific CLI commands which will be returned in the I(rendered) key
        within the result. For state I(rendered) active connection to remote host is
        not required.
      - The state I(gathered) will fetch the running configuration from device and transform
        it into structured data in the format as per the resource module argspec and
        the value is returned in the I(gathered) key within the result.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into JSON format as per the resource module parameters and the
        value is returned in the I(parsed) key within the result. The value of C(running_config)
        option should be the same format as the output of command I(sh running-config
        | section ^route-map) executed on device. For state I(parsed) active
        connection to remote host is not required.
    type: str
"""

EXAMPLES = """
# Using deleted

# Before state:
# -------------
#
# router-ios#sh running-config | section ^route-map
# route-map test_1 deny 10
#  description this is test route
#  match ip next-hop prefix-list test_2_new test_1_new
#  match ip route-source 10
#  match security-group source tag 10 20
#  match local-preference 100 50
#  match mpls-label
# route-map test_1 deny 20
#  match track  105
#  match tag list test_match_tag
#  match route-type level-1
#  match additional-paths advertise-set all group-best
#  match as-path 200 100
#  match ipv6 address test_acl_20
#  continue 100
# route-map test_2 deny 10
#  match security-group source tag 10 20
#  match local-preference 55 105
#  match mpls-label
#  match ipv6 address test_ip_acl
#  match ipv6 next-hop prefix-list test_new
#  match ipv6 route-source route_src_acl
#  set automatic-tag
#  set ip precedence critical
#  set ip address prefix-list 192.0.2.1
#  set aigp-metric 100
#  set extcommunity cost pre-bestpath 10 100
#  set ip df 1
#  set ip next-hop verify-availability 198.51.111.1 100 track 10
#  set ip next-hop recursive global 198.51.110.1

- name: Delete provided Route maps config
  cisco.ios.ios_route_maps:
    config:
      - route_map: test_1
    state: deleted

#  Commands Fired:
#  ---------------
#
#  "commands": [
#         "no route-map test_1"
#     ]

# After state:
# -------------
# router-ios#sh running-config | section ^route-map
# route-map test_2 deny 10
#  match security-group source tag 10 20
#  match local-preference 55 105
#  match mpls-label
#  match ipv6 address test_ip_acl
#  match ipv6 next-hop prefix-list test_new
#  match ipv6 route-source route_src_acl
#  set automatic-tag
#  set ip precedence critical
#  set ip address prefix-list 192.0.2.1
#  set aigp-metric 100
#  set extcommunity cost pre-bestpath 10 100
#  set ip df 1
#  set ip next-hop verify-availability 198.51.111.1 100 track 10
#  set ip next-hop recursive global 198.51.110.1

# Using deleted without any config passed (NOTE: This will delete all Route maps configuration from device)

# Before state:
# -------------
#
# router-ios#sh running-config | section ^route-map
# route-map test_1 deny 10
#  description this is test route
#  match ip next-hop prefix-list test_2_new test_1_new
#  match ip route-source 10
#  match security-group source tag 10 20
#  match local-preference 100 50
#  match mpls-label
# route-map test_1 deny 20
#  match track  105
#  match tag list test_match_tag
#  match route-type level-1
#  match additional-paths advertise-set all group-best
#  match as-path 200 100
#  match ipv6 address test_acl_20
#  continue 100
# route-map test_2 deny 10
#  match security-group source tag 10 20
#  match local-preference 55 105
#  match mpls-label
#  match ipv6 address test_ip_acl
#  match ipv6 next-hop prefix-list test_new
#  match ipv6 route-source route_src_acl
#  set automatic-tag
#  set ip precedence critical
#  set ip address prefix-list 192.0.2.1
#  set aigp-metric 100
#  set extcommunity cost pre-bestpath 10 100
#  set ip df 1
#  set ip next-hop verify-availability 198.51.111.1 100 track 10
#  set ip next-hop recursive global 198.51.110.1

- name: Delete all Route maps config
  cisco.ios.ios_route_maps:
    state: deleted

# Commands Fired:
# ---------------
#
#  "commands": [
#         "no route-map test_1",
#         "no route-map test_2"
#     ]

# After state:
# -------------
# router-ios#sh running-config | section ^route-map
# router-ios#

# Using merged

# Before state:
# -------------
#
# router-ios#sh running-config | section ^route-map
# router-ios#

- name: Merge provided Route maps configuration
  cisco.ios.ios_route_maps:
    config:
      - route_map: test_1
        entries:
          - sequence: 10
            action: deny
            description: this is test route
            match:
              ip:
                next_hop:
                  prefix_lists:
                    - test_1_new
                    - test_2_new
                route_source:
                  acls:
                    - 10
              security_group:
                source:
                  - 10
                  - 20
              local_preference:
                value:
                  - 50
                  - 100
              mpls_label: true
          - sequence: 20
            action: deny
            continue_entry:
              entry_sequence: 100
            match:
              additional_paths:
                all: true
                group_best: true
              as_path:
                acls:
                  - 100
                  - 200
              ipv6:
                address:
                  acl: test_acl_20
              route_type:
                level_1: true
              tag:
                tag_list:
                  - test_match_tag
              track: 105
      - route_map: test_2
        entries:
          - sequence: 10
            action: deny
            match:
              ipv6:
                address:
                  acl: test_ip_acl
                next_hop:
                  prefix_list: test_new
                route_source:
                  acl: route_src_acl
              security_group:
                source:
                  - 10
                  - 20
              local_preference:
                value:
                  - 55
                  - 105
              mpls_label: true
            set:
              aigp_metric:
                value: 100
              automatic_tag: true
              extcommunity:
                cost:
                  id: 10
                  cost_value: 100
                  pre_bestpath: true
              ip:
                address: 192.0.2.1
                df: 1
                next_hop:
                  recursive:
                    global_route: true
                    address: 198.51.110.1
                  verify_availability:
                    address: 198.51.111.1
                    sequence: 100
                    track: 10
                precedence:
                  critical: true
    state: merged

#  Commands Fired:
#  ---------------
#
#   "commands": [
#      "route-map test_2 deny 10",
#      "match security-group source tag 10 20",
#      "match local-preference 55 105",
#      "match mpls-label",
#      "match ipv6 next-hop prefix-list test_new",
#      "match ipv6 route-source route_src_acl",
#      "match ipv6 address test_ip_acl",
#      "set extcommunity cost pre-bestpath 10 100",
#      "set ip df 1",
#      "set ip next-hop recursive global 198.51.110.1",
#      "set ip next-hop verify-availability 198.51.111.1 100 track 10",
#      "set ip precedence critical",
#      "set ip address prefix-list 192.0.2.1",
#      "set automatic-tag",
#      "set aigp-metric 100",
#      "route-map test_1 deny 20",
#      "continue 100",
#      "match track 105",
#      "match tag list test_match_tag",
#      "match ipv6 address test_acl_20",
#      "match route-type level-1",
#      "match as-path 200 100",
#      "match additional-paths advertise-set all group-best",
#      "route-map test_1 deny 10",
#      "description this is test route",
#      "match security-group source tag 10 20",
#      "match ip next-hop prefix-list test_2_new test_1_new",
#      "match ip route-source 10",
#      "match local-preference 100 50",
#      "match mpls-label"
#     ]

# After state:
# -------------
#
# router-ios#sh running-config | section ^route-map
# route-map test_1 deny 10
#  description this is test route
#  match ip next-hop prefix-list test_2_new test_1_new
#  match ip route-source 10
#  match security-group source tag 10 20
#  match local-preference 100 50
#  match mpls-label
# route-map test_1 deny 20
#  match track  105
#  match tag list test_match_tag
#  match route-type level-1
#  match additional-paths advertise-set all group-best
#  match as-path 200 100
#  match ipv6 address test_acl_20
#  continue 100
# route-map test_2 deny 10
#  match security-group source tag 10 20
#  match local-preference 55 105
#  match mpls-label
#  match ipv6 address test_ip_acl
#  match ipv6 next-hop prefix-list test_new
#  match ipv6 route-source route_src_acl
#  set automatic-tag
#  set ip precedence critical
#  set ip address prefix-list 192.0.2.1
#  set aigp-metric 100
#  set extcommunity cost pre-bestpath 10 100
#  set ip df 1
#  set ip next-hop verify-availability 198.51.111.1 100 track 10
#  set ip next-hop recursive global 198.51.110.1

# Using overridden

# Before state:
# -------------
#
# router-ios#sh running-config | section ^route-map
# route-map test_1 deny 10
#  description this is test route
#  match ip next-hop prefix-list test_2_new test_1_new
#  match ip route-source 10
#  match security-group source tag 10 20
#  match local-preference 100 50
#  match mpls-label
# route-map test_1 deny 20
#  match track  105
#  match tag list test_match_tag
#  match route-type level-1
#  match additional-paths advertise-set all group-best
#  match as-path 200 100
#  match ipv6 address test_acl_20
#  continue 100
# route-map test_2 deny 10
#  match security-group source tag 10 20
#  match local-preference 55 105
#  match mpls-label
#  match ipv6 address test_ip_acl
#  match ipv6 next-hop prefix-list test_new
#  match ipv6 route-source route_src_acl
#  set automatic-tag
#  set ip precedence critical
#  set ip address prefix-list 192.0.2.1
#  set aigp-metric 100
#  set extcommunity cost pre-bestpath 10 100
#  set ip df 1
#  set ip next-hop verify-availability 198.51.111.1 100 track 10
#  set ip next-hop recursive global 198.51.110.1

- name: Override provided Route maps configuration
  cisco.ios.ios_route_maps:
    config:
      - route_map: test_1
        entries:
          - sequence: 10
            action: deny
            description: this is override route
            match:
              ip:
                next_hop:
                  acls:
                    - 10
                    - test1_acl
                flowspec:
                  dest_pfx: true
                  acls:
                    - test_acl_1
                    - test_acl_2
              length:
                minimum: 10
                maximum: 100
              metric:
                value: 10
                external: true
              security_group:
                source:
                  - 10
                  - 20
              mpls_label: true
            set:
              extcommunity:
                vpn_distinguisher:
                  address: 192.0.2.1:12
                  additive: true
              metric:
                metric_value: 100
                deviation: plus
                eigrp_delay: 100
                metric_reliability: 10
                metric_bandwidth: 20
                mtu: 30
      - route_map: test_override
        entries:
          - sequence: 10
            action: deny
            match:
              ipv6:
                address:
                  acl: test_acl
                next_hop:
                  prefix_list: test_new
                route_source:
                  acl: route_src_acl
              security_group:
                source:
                  - 15
                  - 20
              local_preference:
                value:
                  - 105
                  - 110
              mpls_label: true
            set:
              aigp_metric:
                value: 100
              automatic_tag: true
              extcommunity:
                cost:
                  id: 10
                  cost_value: 100
                  pre_bestpath: true
              ip:
                address: 192.0.2.1
                df: 1
                next_hop:
                  recursive:
                    global_route: true
                    address: 198.110.51.1
                  verify_availability:
                    address: 198.110.51.2
                    sequence: 100
                    track: 10
                precedence:
                  critical: true
    state: overridden

# Commands Fired:
# ---------------
#
#  "commands": [
#         "no route-map test_2",
#         "route-map test_override deny 10",
#         "match security-group source tag 15 20",
#         "match local-preference 110 105",
#         "match mpls-label",
#         "match ipv6 next-hop prefix-list test_new",
#         "match ipv6 route-source route_src_acl",
#         "match ipv6 address test_acl",
#         "set extcommunity cost pre-bestpath 10 100",
#         "set ip df 1",
#         "set ip next-hop recursive global 198.110.51.1",
#         "set ip next-hop verify-availability 198.110.51.2 100 track 10",
#         "set ip precedence critical",
#         "set ip address prefix-list 192.0.2.1",
#         "set automatic-tag",
#         "set aigp-metric 100",
#         "route-map test_1 deny 10",
#         "no description this is test route",
#         "description this is override route",
#         "match ip flowspec dest-pfx test_acl_1 test_acl_2",
#         "no match ip next-hop prefix-list test_2_new test_1_new",
#         "match ip next-hop test1_acl 10",
#         "no match ip route-source 10",
#         "match metric external 10",
#         "match length 10 100",
#         "no match local-preference 100 50",
#         "set extcommunity vpn-distinguisher 192.0.2.1:12 additive",
#         "set metric 100 +100 10 20 30",
#         "no route-map test_1 deny 20"
#     ]

# After state:
# -------------
#
# router-ios#sh running-config | section ^route-map
# route-map test_override deny 10
#  match security-group source tag 15 20
#  match local-preference 110 105
#  match mpls-label
#  match ipv6 address test_acl
#  match ipv6 next-hop prefix-list test_new
#  match ipv6 route-source route_src_acl
#  set automatic-tag
#  set ip precedence critical
#  set ip address prefix-list 192.0.2.1
#  set aigp-metric 100
#  set extcommunity cost pre-bestpath 10 100
#  set ip df 1
#  set ip next-hop verify-availability 198.110.51.2 100 track 10
#  set ip next-hop recursive global 198.110.51.1
# route-map test_1 deny 10
#  description this is override route
#  match ip flowspec dest-pfx test_acl_1 test_acl_2
#  match ip next-hop test1_acl 10
#  match security-group source tag 10 20
#  match metric external 10
#  match mpls-label
#  match length 10 100
#  set metric 100 +100 10 20 30
#  set extcommunity vpn-distinguisher 192.0.2.1:12 additive

# Using replaced

# Before state:
# -------------
#
# router-ios#sh running-config | section ^route-map
# route-map test_1 deny 10
#  description this is test route
#  match ip next-hop prefix-list test_2_new test_1_new
#  match ip route-source 10
#  match security-group source tag 10 20
#  match local-preference 100 50
#  match mpls-label
# route-map test_1 deny 20
#  match track  105
#  match tag list test_match_tag
#  match route-type level-1
#  match additional-paths advertise-set all group-best
#  match as-path 200 100
#  match ipv6 address test_acl_20
#  continue 100
# route-map test_2 deny 10
#  match security-group source tag 10 20
#  match local-preference 55 105
#  match mpls-label
#  match ipv6 address test_ip_acl
#  match ipv6 next-hop prefix-list test_new
#  match ipv6 route-source route_src_acl
#  set automatic-tag
#  set ip precedence critical
#  set ip address prefix-list 192.0.2.1
#  set aigp-metric 100
#  set extcommunity cost pre-bestpath 10 100
#  set ip df 1
#  set ip next-hop verify-availability 198.51.111.1 100 track 10
#  set ip next-hop recursive global 198.51.110.1

- name: Replaced provided Route maps configuration
  cisco.ios.ios_route_maps:
    config:
      - route_map: test_1
        entries:
          - sequence: 10
            action: deny
            description: this is replaced route
            match:
              ip:
                next_hop:
                  acls:
                    - 10
                    - test1_acl
                flowspec:
                  dest_pfx: true
                  acls:
                    - test_acl_1
                    - test_acl_2
              length:
                minimum: 10
                maximum: 100
              metric:
                value: 10
                external: true
              security_group:
                source:
                  - 10
                  - 20
              mpls_label: true
            set:
              extcommunity:
                vpn_distinguisher:
                  address: 192.0.2.1:12
                  additive: true
              metric:
                metric_value: 100
                deviation: plus
                eigrp_delay: 100
                metric_reliability: 10
                metric_bandwidth: 20
                mtu: 30
      - route_map: test_replaced
        entries:
          - sequence: 10
            action: deny
            match:
              ipv6:
                address:
                  acl: test_acl
                next_hop:
                  prefix_list: test_new
                route_source:
                  acl: route_src_acl
              security_group:
                source:
                  - 15
                  - 20
              local_preference:
                value:
                  - 105
                  - 110
              mpls_label: true
            set:
              aigp_metric:
                value: 100
              automatic_tag: true
              extcommunity:
                cost:
                  id: 10
                  cost_value: 100
                  pre_bestpath: true
              ip:
                address: 192.0.2.1
                df: 1
                next_hop:
                  recursive:
                    global_route: true
                    address: 198.110.51.1
                  verify_availability:
                    address: 198.110.51.2
                    sequence: 100
                    track: 10
                precedence:
                  critical: true
    state: replaced

# Commands Fired:
# ---------------
#  "commands": [
#         "route-map test_replaced deny 10",
#         "match security-group source tag 15 20",
#         "match local-preference 110 105",
#         "match mpls-label",
#         "match ipv6 next-hop prefix-list test_new",
#         "match ipv6 route-source route_src_acl",
#         "match ipv6 address test_acl",
#         "set extcommunity cost pre-bestpath 10 100",
#         "set ip df 1",
#         "set ip next-hop recursive global 198.110.51.1",
#         "set ip next-hop verify-availability 198.110.51.2 100 track 10",
#         "set ip precedence critical",
#         "set ip address prefix-list 192.0.2.1",
#         "set automatic-tag",
#         "set aigp-metric 100",
#         "route-map test_1 deny 10",
#         "no description this is test route",
#         "description this is replaced route",
#         "match ip flowspec dest-pfx test_acl_1 test_acl_2",
#         "no match ip next-hop prefix-list test_2_new test_1_new",
#         "match ip next-hop test1_acl 10",
#         "no match ip route-source 10",
#         "match metric external 10",
#         "match length 10 100",
#         "no match local-preference 100 50",
#         "set extcommunity vpn-distinguisher 192.0.2.1:12 additive",
#         "set metric 100 +100 10 20 30",
#         "no route-map test_1 deny 20"
#     ]

# After state:
# -------------
#
# router-ios#sh running-config | section ^route-map
# route-map test_replaced deny 10
#  match security-group source tag 15 20
#  match local-preference 110 105
#  match mpls-label
#  match ipv6 address test_acl
#  match ipv6 next-hop prefix-list test_new
#  match ipv6 route-source route_src_acl
#  set automatic-tag
#  set ip precedence critical
#  set ip address prefix-list 192.0.2.1
#  set aigp-metric 100
#  set extcommunity cost pre-bestpath 10 100
#  set ip df 1
#  set ip next-hop verify-availability 198.110.51.2 100 track 10
#  set ip next-hop recursive global 198.110.51.1
# route-map test_1 deny 10
#  description this is replaced route
#  match ip flowspec dest-pfx test_acl_1 test_acl_2
#  match ip next-hop test1_acl 10
#  match security-group source tag 10 20
#  match metric external 10
#  match mpls-label
#  match length 10 100
#  set metric 100 +100 10 20 30
#  set extcommunity vpn-distinguisher 192.0.2.1:12 additive
# route-map test_2 deny 10
#  match security-group source tag 10 20
#  match local-preference 55 105
#  match mpls-label
#  match ipv6 address test_ip_acl
#  match ipv6 next-hop prefix-list test_new
#  match ipv6 route-source route_src_acl
#  set automatic-tag
#  set ip precedence critical
#  set ip address prefix-list 192.0.2.1
#  set aigp-metric 100
#  set extcommunity cost pre-bestpath 10 100
#  set ip df 1
#  set ip next-hop verify-availability 198.51.111.1 100 track 10
#  set ip next-hop recursive global 198.51.110.1

# Using Gathered

# Before state:
# -------------
#
# router-ios#sh running-config | section ^route-map
# route-map test_1 deny 10
#  description this is test route
#  match ip next-hop prefix-list test_2_new test_1_new
#  match ip route-source 10
#  match security-group source tag 10 20
#  match local-preference 100 50
#  match mpls-label
# route-map test_1 deny 20
#  match track  105
#  match tag list test_match_tag
#  match route-type level-1
#  match additional-paths advertise-set all group-best
#  match as-path 200 100
#  match ipv6 address test_acl_20
#  continue 100
# route-map test_2 deny 10
#  match security-group source tag 10 20
#  match local-preference 55 105
#  match mpls-label
#  match ipv6 address test_ip_acl
#  match ipv6 next-hop prefix-list test_new
#  match ipv6 route-source route_src_acl
#  set automatic-tag
#  set ip precedence critical
#  set ip address prefix-list 192.0.2.1
#  set aigp-metric 100
#  set extcommunity cost pre-bestpath 10 100
#  set ip df 1
#  set ip next-hop verify-availability 198.51.111.1 100 track 10
#  set ip next-hop recursive global 198.51.110.1

- name: Gather Route maps provided configurations
  cisco.ios.ios_route_maps:
    config:
    state: gathered

# Module Execution Result:
# ------------------------
#
# "gathered": [
#         {
#             "entries": [
#                 {
#                     "action": "deny",
#                     "description": "this is test route",
#                     "match": {
#                         "ip": {
#                             "next_hop": {
#                                 "prefix_lists": [
#                                     "test_2_new",
#                                     "test_1_new"
#                                 ]
#                             },
#                             "route_source": {
#                                 "acls": [
#                                     "10"
#                                 ]
#                             }
#                         },
#                         "local_preference": {
#                             "value": [
#                                 "100",
#                                 "50"
#                             ]
#                         },
#                         "mpls_label": true,
#                         "security_group": {
#                             "source": [
#                                 10,
#                                 20
#                             ]
#                         }
#                     },
#                     "sequence": 10
#                 },
#                 {
#                     "action": "deny",
#                     "continue_entry": {
#                         "entry_sequence": 100
#                     },
#                     "match": {
#                         "additional_paths": {
#                             "all": true,
#                             "group_best": true
#                         },
#                         "as_path": {
#                             "acls": [
#                                 200,
#                                 100
#                             ]
#                         },
#                         "ipv6": {
#                             "address": {
#                                 "acl": "test_acl_20"
#                             }
#                         },
#                         "route_type": {
#                             "external": {
#                                 "set": true
#                             },
#                             "level_1": true,
#                             "nssa_external": {
#                                 "set": true
#                             }
#                         },
#                         "tag": {
#                             "tag_list": [
#                                 "test_match_tag"
#                             ]
#                         },
#                         "track": 105
#                     },
#                     "sequence": 20
#                 }
#             ],
#             "route_map": "test_1"
#         },
#         {
#             "entries": [
#                 {
#                     "action": "deny",
#                     "match": {
#                         "ipv6": {
#                             "address": {
#                                 "acl": "test_ip_acl"
#                             },
#                             "next_hop": {
#                                 "prefix_list": "test_new"
#                             },
#                             "route_source": {
#                                 "acl": "route_src_acl"
#                             }
#                         },
#                         "local_preference": {
#                             "value": [
#                                 "55",
#                                 "105"
#                             ]
#                         },
#                         "mpls_label": true,
#                         "security_group": {
#                             "source": [
#                                 10,
#                                 20
#                             ]
#                         }
#                     },
#                     "sequence": 10,
#                     "set": {
#                         "aigp_metric": {
#                             "value": 100
#                         },
#                         "automatic_tag": true,
#                         "extcommunity": {
#                             "cost": {
#                                 "cost_value": 100,
#                                 "id": "10",
#                                 "pre_bestpath": true
#                             }
#                         },
#                         "ip": {
#                             "address": "192.0.2.1",
#                             "df": 1,
#                             "next_hop": {
#                                 "recursive": {
#                                     "address": "198.51.110.1",
#                                     "global_route": true
#                                 },
#                                 "verify_availability": {
#                                     "address": "198.51.111.1",
#                                     "sequence": 100,
#                                     "track": 10
#                                 }
#                             },
#                             "precedence": {
#                                 "critical": true
#                             }
#                         }
#                     }
#                 }
#             ],
#             "route_map": "test_2"
#         }
#     ]

# After state:
# ------------
#
# router-ios#sh running-config | section ^route-map
# route-map test_1 deny 10
#  description this is test route
#  match ip next-hop prefix-list test_2_new test_1_new
#  match ip route-source 10
#  match security-group source tag 10 20
#  match local-preference 100 50
#  match mpls-label
# route-map test_1 deny 20
#  match track  105
#  match tag list test_match_tag
#  match route-type level-1
#  match additional-paths advertise-set all group-best
#  match as-path 200 100
#  match ipv6 address test_acl_20
#  continue 100
# route-map test_2 deny 10
#  match security-group source tag 10 20
#  match local-preference 55 105
#  match mpls-label
#  match ipv6 address test_ip_acl
#  match ipv6 next-hop prefix-list test_new
#  match ipv6 route-source route_src_acl
#  set automatic-tag
#  set ip precedence critical
#  set ip address prefix-list 192.0.2.1
#  set aigp-metric 100
#  set extcommunity cost pre-bestpath 10 100
#  set ip df 1
#  set ip next-hop verify-availability 198.51.111.1 100 track 10
#  set ip next-hop recursive global 198.51.110.1

# Using Rendered

- name: Render the commands for provided  configuration
  cisco.ios.ios_route_maps:
    config:
      - route_map: test_1
        entries:
          - sequence: 10
            action: deny
            description: this is test route
            match:
              ip:
                next_hop:
                  prefix_lists:
                    - test_1_new
                    - test_2_new
                route_source:
                  acls:
                    - 10
              security_group:
                source:
                  - 10
                  - 20
              local_preference:
                value:
                  - 50
                  - 100
              mpls_label: true
          - sequence: 20
            action: deny
            continue_entry:
              entry_sequence: 100
            match:
              additional_paths:
                all: true
                group_best: true
              as_path:
                acls:
                  - 100
                  - 200
              ipv6:
                address:
                  acl: test_acl_20
              route_type:
                level_1: true
              tag:
                tag_list:
                  - test_match_tag
              track: 105
      - route_map: test_2
        entries:
          - sequence: 10
            action: deny
            match:
              ipv6:
                address:
                  acl: test_ip_acl
                next_hop:
                  prefix_list: test_new
                route_source:
                  acl: route_src_acl
              security_group:
                source:
                  - 10
                  - 20
              local_preference:
                value:
                  - 55
                  - 105
              mpls_label: true
            set:
              aigp_metric:
                value: 100
              automatic_tag: true
              extcommunity:
                cost:
                  id: 10
                  cost_value: 100
                  pre_bestpath: true
              ip:
                address: 192.0.2.1
                df: 1
                next_hop:
                  recursive:
                    global_route: true
                    address: 198.51.110.1
                  verify_availability:
                    address: 198.51.111.1
                    sequence: 100
                    track: 10
                precedence:
                  critical: true
    state: rendered

# Module Execution Result:
# ------------------------
#
#  "rendered": [
#      "route-map test_2 deny 10",
#      "match security-group source tag 10 20",
#      "match local-preference 55 105",
#      "match mpls-label",
#      "match ipv6 next-hop prefix-list test_new",
#      "match ipv6 route-source route_src_acl",
#      "match ipv6 address test_ip_acl",
#      "set extcommunity cost pre-bestpath 10 100",
#      "set ip df 1",
#      "set ip next-hop recursive global 198.51.110.1",
#      "set ip next-hop verify-availability 198.51.111.1 100 track 10",
#      "set ip precedence critical",
#      "set ip address prefix-list 192.0.2.1",
#      "set automatic-tag",
#      "set aigp-metric 100",
#      "route-map test_1 deny 20",
#      "continue 100",
#      "match track 105",
#      "match tag list test_match_tag",
#      "match ipv6 address test_acl_20",
#      "match route-type level-1",
#      "match as-path 200 100",
#      "match additional-paths advertise-set all group-best",
#      "route-map test_1 deny 10",
#      "description this is test route",
#      "match security-group source tag 10 20",
#      "match ip next-hop prefix-list test_2_new test_1_new",
#      "match ip route-source 10",
#      "match local-preference 100 50",
#      "match mpls-label"
#     ]

# Using Parsed

# File: parsed.cfg
# ----------------
#
# route-map test_1 deny 10
#  description this is test route
#  match ip next-hop prefix-list test_2_new test_1_new
#  match ip route-source 10
#  match security-group source tag 10 20
#  match local-preference 100 50
#  match mpls-label
# route-map test_1 deny 20
#  match track  105
#  match tag list test_match_tag
#  match route-type level-1
#  match additional-paths advertise-set all group-best
#  match as-path 200 100
#  match ipv6 address test_acl_20
#  continue 100
# route-map test_2 deny 10
#  match security-group source tag 10 20
#  match local-preference 55 105
#  match mpls-label
#  match ipv6 address test_ip_acl
#  match ipv6 next-hop prefix-list test_new
#  match ipv6 route-source route_src_acl
#  set automatic-tag
#  set ip precedence critical
#  set ip address prefix-list 192.0.2.1
#  set aigp-metric 100
#  set extcommunity cost pre-bestpath 10 100
#  set ip df 1
#  set ip next-hop verify-availability 198.51.111.1 100 track 10
#  set ip next-hop recursive global 198.51.110.1

- name: Parse the provided configuration with the existing running configuration
  cisco.ios.ios_route_maps:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Module Execution Result:
# ------------------------
#
# "parsed": [
#         {
#             "entries": [
#                 {
#                     "action": "deny",
#                     "description": "this is test route",
#                     "match": {
#                         "ip": {
#                             "next_hop": {
#                                 "prefix_lists": [
#                                     "test_2_new",
#                                     "test_1_new"
#                                 ]
#                             },
#                             "route_source": {
#                                 "acls": [
#                                     "10"
#                                 ]
#                             }
#                         },
#                         "local_preference": {
#                             "value": [
#                                 "100",
#                                 "50"
#                             ]
#                         },
#                         "mpls_label": true,
#                         "security_group": {
#                             "source": [
#                                 10,
#                                 20
#                             ]
#                         }
#                     },
#                     "sequence": 10
#                 },
#                 {
#                     "action": "deny",
#                     "continue_entry": {
#                         "entry_sequence": 100
#                     },
#                     "match": {
#                         "additional_paths": {
#                             "all": true,
#                             "group_best": true
#                         },
#                         "as_path": {
#                             "acls": [
#                                 200,
#                                 100
#                             ]
#                         },
#                         "ipv6": {
#                             "address": {
#                                 "acl": "test_acl_20"
#                             }
#                         },
#                         "route_type": {
#                             "external": {
#                                 "set": true
#                             },
#                             "level_1": true,
#                             "nssa_external": {
#                                 "set": true
#                             }
#                         },
#                         "tag": {
#                             "tag_list": [
#                                 "test_match_tag"
#                             ]
#                         },
#                         "track": 105
#                     },
#                     "sequence": 20
#                 }
#             ],
#             "route_map": "test_1"
#         },
#         {
#             "entries": [
#                 {
#                     "action": "deny",
#                     "match": {
#                         "ipv6": {
#                             "address": {
#                                 "acl": "test_ip_acl"
#                             },
#                             "next_hop": {
#                                 "prefix_list": "test_new"
#                             },
#                             "route_source": {
#                                 "acl": "route_src_acl"
#                             }
#                         },
#                         "local_preference": {
#                             "value": [
#                                 "55",
#                                 "105"
#                             ]
#                         },
#                         "mpls_label": true,
#                         "security_group": {
#                             "source": [
#                                 10,
#                                 20
#                             ]
#                         }
#                     },
#                     "sequence": 10,
#                     "set": {
#                         "aigp_metric": {
#                             "value": 100
#                         },
#                         "automatic_tag": true,
#                         "extcommunity": {
#                             "cost": {
#                                 "cost_value": 100,
#                                 "id": "10",
#                                 "pre_bestpath": true
#                             }
#                         },
#                         "ip": {
#                             "address": "192.0.2.1",
#                             "df": 1,
#                             "next_hop": {
#                                 "recursive": {
#                                     "address": "198.51.110.1",
#                                     "global_route": true
#                                 },
#                                 "verify_availability": {
#                                     "address": "198.51.111.1",
#                                     "sequence": 100,
#                                     "track": 10
#                                 }
#                             },
#                             "precedence": {
#                                 "critical": true
#                             }
#                         }
#                     }
#                 }
#             ],
#             "route_map": "test_2"
#         }
#     ]
"""

RETURN = """
before:
  description: The configuration prior to the model invocation.
  returned: always
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
  type: list
after:
  description: The resulting configuration model invocation.
  returned: when changed
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
  type: list
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample: ['route-map test_1 deny 10', 'description this is test route', 'match ip route-source 10', 'match track  105']
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.route_maps.route_maps import (
    Route_mapsArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.config.route_maps.route_maps import (
    Route_maps,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Route_mapsArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Route_maps(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()
